unit FoxCheckControls;

// Part of the FoxControls package suite
// by A Fox Consulting & Design
// see FoxControls.txt or visit
// www.afox-consulting.com for more information.
//
// REMEMBER TO INCLUDE FoxClasses IN YOUR UNIT IF YOU NEED
// ACCESS TO THE SPECIAL ENUMERATED PROPERTY CLASSES USED
// BY THE FOXCONTROLS!
//
// Note: Strictly speaking these controls are not 100%
// transparent in that any other graphic controls they overlap
// will not show through. There is a way around this by
// cycling through and re-painting all visual controls that
// intersect them ((FoxEdit use to do this, for example) but
// it never really worked very well and looked pretty messy
// with situations like transparent checkboxes inside of
// transparent group boxes. That's why I stuck with this
// simpler method. -- FOX 5/7/10

{$I Compilers.inc}

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, FoxClasses;

{$R FoxControls.res}

type
  TFoxCheckBox = class(TCustomControl)
  private
    R,CaptRect: TRect;
    TransColor: Tcolor;
    CurrentColor : TColor;
    CurrentFontColor : TColor;
    CurrentBorder : TFoxBorder;
    ButtonReleased: Boolean;
    IsHot: Boolean;
    FAlignment: TLeftRight;
    FAutoSize: Boolean;
    FBorder: TFoxBorder;
    FBorderColor:TColor;
    FBorderVisible: TFoxBorderVisible;
    FChecked: Boolean;
    FCheckColor: TColor;
    FCheckStyle: TFoxCheckStyle;
    FCheckType: TFoxCheckType;
    FCtl3D: Boolean;
    FFocusHighlight: boolean;
    FGlyph: TBitmap;
    FHighlightColor: TColor;
    FHintTwo: String;
    FHotTrack: boolean;
    FHotTrackFontColor: TColor;
    FOnMouseOver: TOnMouseOverEvent;
    FOnMouseOut: TOnMouseOutEvent;
    FReadOnly: Boolean;
    FTransparent: Boolean;
    FTransparentCenter: Boolean;
    FVersion: String;
    FWordWrap: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetAutoSize(Value: Boolean);
    procedure SetBorder(Value: TFoxBorder);
    procedure SetBorderColor(Value: TColor);
    procedure SetBorderVisible(Value: TFoxBorderVisible);
    procedure SetChecked(Value: Boolean);
    procedure SetCheckColor(Value: TColor);
    procedure SetCheckStyle(Value: TFoxCheckStyle);
    procedure SetCheckType(Value: TFoxCheckType);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetHintTwo(Value: String);
    procedure SetHotTrack(Value: Boolean);
    procedure SetHotTrackFontColor(const Value: TColor);
    procedure SetReadOnly(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentCenter(Value: Boolean);
    procedure SetVersion(const Value: String);
    procedure SetWordWrap(Value: Boolean);
    procedure DrawBorder;
    procedure DrawCaption;
    procedure DrawCaptionDisabled;
    procedure DrawCheckbox;
    procedure PickGlyph;
    procedure ColorGlyph;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter(var AMsg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var AMsg: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CMVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
    procedure WMLButtonDblClk (var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure Click; override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
     X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TLeftRight read FAlignment write SetAlignment;
    //property AllowGrayed;  -- We do not support this feature
    property Anchors;
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property BiDiMode;
    property Border: TFoxBorder read FBorder write SetBorder;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BorderVisible: TFoxBorderVisible read FBorderVisible write SetBorderVisible;
    property Caption;
    property Checked: Boolean read FChecked write SetChecked;
    property CheckColor: TColor read FCheckColor write SetCheckColor;
    property CheckStyle: TFoxCheckStyle read FCheckStyle write SetCheckStyle;
    property CheckType: TFoxCheckType read FCheckType write SetCheckType;
    property Color;
    property Constraints;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property Font;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property HotTrackFontColor: TColor read FHotTrackFontColor write SetHotTrackFontColor;
    property HintSecondLine: String read FHintTwo write SetHintTwo;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter: TOnMouseOverEvent read FOnMouseOver write FOnMouseOver;
    property OnMouseExit: TOnMouseOutEvent read FOnMouseOut write FOnMouseOut;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property ParentBiDiMode;
//    property ParentCtl3D;
    property ParentFont;
    property ParentColor;
    property PopupMenu;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly;
    property ShowHint;
    property TabStop;
    property TabOrder;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property TransparentCenter: Boolean read FTransparentCenter write SetTransparentCenter;
    property Version: String read FVersion write SetVersion;
    property Visible;
    property WordWrap: Boolean read FWordWrap write SetWordWrap;
    {$IFDEF DELPHI_5_UP}
      property Action;
      property OnContextPopup;
    {$ENDIF}
  end;

type
  TFoxGroupCheck = class(TFoxCheckBox)
  protected
    procedure Click; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  end;

type
  TFoxRadioButton = class(TCustomControl)
  private
    R,CaptRect:TRect;
    CurrentColor : TColor;
    CurrentFontColor : TColor;
    CurrentBorder : TFoxBorder;
    ButtonReleased: Boolean;
    PrevCheckState: Boolean;
    IsHot: Boolean;
    FAlignment: TLeftRight;
    FAutoSize: boolean;
    FBorder: TFoxBorder;
    FBorderColor: TColor;
    FBorderVisible: TFoxBorderVisible;
    FChecked: Boolean;
    FCheckColor: TColor;
    FCheckStyle: TFoxCheckStyle;
    FCtl3D: Boolean;
    FFocusHighlight: Boolean;
    FGlyph: TBitmap;
    FGroupIndex: Byte;
    FHighlightColor: TColor;
    FHintTwo: String;
    FHotTrack: Boolean;
    FHotTrackFontColor: TColor;
    FRaised: Boolean;
    FReadOnly: Boolean;
    FOnMouseOver: TOnMouseOverEvent;
    FOnMouseOut: TOnMouseOutEvent;
    FTransparent: Boolean;
    FTransparentCenter: Boolean;
    FVersion: string;
    FWordWrap: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetAutoSize(Value: Boolean);
    procedure SetBorder(Value: TFoxBorder);
    procedure SetBorderColor(Value: TColor);
    procedure SetBorderVisible(Value: TFoxBorderVisible);
    procedure SetCheckStyle(Value: TFoxCheckStyle);
    procedure SetChecked(Value: Boolean);
    procedure SetCheckColor(Value: TColor);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetHintTwo(Value: string);
    procedure SetHotTrack(Value: Boolean);
    procedure SetHotTrackFontColor(const Value: TColor);
    procedure SetRaised(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentCenter(Value: Boolean);
    procedure SetVersion(const Value: string);
    procedure SetWordWrap(Value: Boolean);
    procedure TurnSiblingsOff;
    procedure DrawBorder;
    procedure DrawCaption;
    procedure DrawCaptionDisabled;
    procedure DrawButton;
    procedure PickGlyph;
    procedure ColorGlyph;
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter(var AMsg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var AMsg: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CMVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
    procedure WMLButtonDblClk (var Message: TWMLButtonDown); message WM_LBUTTONDBLCLK;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure Click; override;
    procedure Paint; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
     X, Y: Integer); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TLeftRight read FAlignment write SetAlignment;
    property Anchors;
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property BiDiMode;
    property Border: TFoxBorder read FBorder write SetBorder;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BorderVisible: TFoxBorderVisible read FBorderVisible write SetBorderVisible;
    property Caption;
    property Checked: Boolean read FChecked write SetChecked;
    property CheckColor: TColor read FCheckColor write SetCheckColor;
    property CheckStyle: TFoxCheckStyle read FCheckStyle write SetCheckStyle;
    property Color;
    property Constraints;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property GroupIndex: Byte read FGroupIndex write FGroupIndex;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property HintSecondLine: String read FHintTwo write SetHintTwo;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property HotTrackFontColor: TColor read FHotTrackFontColor write SetHotTrackFontColor;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter: TOnMouseOverEvent read FOnMouseOver write FOnMouseOver;
    property OnMouseExit: TOnMouseOutEvent read FOnMouseOut write FOnMouseOut;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property ParentBiDiMode;
//    property ParentCtl3D;
    property ParentFont;
    property ParentColor;
    property PopupMenu;
    property Raised: Boolean read FRaised write SetRaised;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly;
    property ShowHint;
    property TabStop;
    property TabOrder;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property TransparentCenter: Boolean read FTransparentCenter write SetTransparentCenter;
    property Version: string read FVersion write SetVersion;
    property Visible;
    property WordWrap: Boolean read FWordWrap write SetWordWrap;
    {$IFDEF DELPHI_5_UP}
      property Action;
      property OnContextPopup;
    {$ENDIF}
  end;

type
  TFoxRadioGroup = class(TCustomGroupBox)
  private
    FAlignment: TLeftRight;
    FAutoSize: Boolean;
    FBorder: TFoxBorder;
    FBorderColor: TColor;
    FBorderVisible: TFoxBorderVisible;
    FButtons: TList;
    FCheckColor: TColor;
    FCheckStyle: TFoxCheckStyle;
    FColumns: Integer;
    FCtl3D: Boolean;
    FFocusHighlight: Boolean;
    FHighlightColor: TColor;
    FHotTrack: Boolean;
    FHotTrackFontColor: TColor;
    FItemIndex: Integer;
    FItems: TStrings;
    FRaised: Boolean;
    FReading: Boolean;
    FReadOnly: Boolean;
    FUpdating: Boolean;
    FTransparent: Boolean;
    FTransparentCenter: Boolean;
    FVersion: String;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetAutoSize(Value: boolean);
    procedure SetBorder(Value: TFoxBorder);
    procedure SetBorderColor(Value: TColor);
    procedure SetBorderVisible(Value: TFoxBorderVisible);
    procedure SetButtonCount(Value: Integer);
    procedure SetCheckStyle(Value: TFoxCheckStyle);
    procedure SetCheckColor(Value: TColor);
    procedure SetColumns(Value: Integer);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetHotTrack(Value: Boolean);
    procedure SetHotTrackFontColor(const Value: TColor);
    procedure SetItemIndex(Value: Integer);
    procedure SetItems(Value: TStrings);
    procedure SetRaised(Value: Boolean);
    procedure SetReadOnly(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentCenter(Value: Boolean);
    procedure SetVersion(const Value: string);
    procedure DrawFrame(R : TRect; C : TColor);
    procedure ArrangeButtons;
    procedure ButtonClick(Sender: TObject);
    procedure ItemsChange(Sender: TObject);
    procedure UpdateButtons;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure Loaded; override;
    procedure Paint; override;
    procedure ReadState(Reader: TReader); override;
    function CanModify: Boolean; virtual;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FlipChildren(AllLevels: Boolean); override;
  published
    property Align;
    property Alignment: TLeftRight read FAlignment write SetAlignment;
    property Anchors;
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property BiDiMode;
    property Border: TFoxBorder read FBorder write SetBorder;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property BorderVisible: TFoxBorderVisible read FBorderVisible write SetBorderVisible;
    property Caption;
    property CheckColor: TColor read FCheckColor write SetCheckColor;
    property CheckStyle: TFoxCheckStyle read FCheckStyle write SetCheckStyle;
    property Color;
    property Columns: Integer read FColumns write SetColumns default 1;
    property Constraints;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property Font;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property HotTrackFontColor: TColor read FHotTrackFontColor write SetHotTrackFontColor;
    property ItemIndex: Integer read FItemIndex write SetItemIndex default -1;
    property Items: TStrings read FItems write SetItems;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnStartDock;
    property OnStartDrag;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property Raised: Boolean read FRaised write SetRaised;
    property ReadOnly: Boolean read FReadOnly write SetReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property TransparentCenter: Boolean read FTransparentCenter write SetTransparentCenter;
    property Version: String read FVersion write SetVersion;
    property Visible;
    {$IFDEF DELPHI_5_UP}
      property OnContextPopup;
    {$ENDIF}
  end;

type
  TFoxGroupButton = class(TFoxRadioButton)
  private
    FInClick: Boolean;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
  public
    constructor InternalCreate(RadioGroup: TFoxRadioGroup);
    destructor Destroy; override;
  end;

type
  TFoxGroupBox = class(TGroupBox)
  private
    FCtl3D: Boolean;
    FTransparent: Boolean;
    FVersion: String;
    procedure SetCtl3D(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetVersion(const Value: string);
    procedure DrawFrame(R : TRect; C : TColor);
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
    procedure RepaintChildren;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Version: String read FVersion write SetVersion;
  end;

type
  TFoxGroupBoxWithCheck = class(TGroupBox)
  private
    FCheckBox: TFoxGroupCheck;
    FCtl3D: Boolean;
    FOnMouseOver: TOnMouseOverEvent;
    FOnMouseOut: TOnMouseOutEvent;
    FTransparent: Boolean;
    FVersion: String;
    function GetAlignment: TLeftRight;
    function GetChecked: Boolean;
    function GetCheckColor: TColor;
    function GetCheckStyle: TFoxCheckStyle;
    function GetCheckType: TFoxCheckType;
    function GetFocusHighlight: Boolean;
    function GetHighlightColor: TColor;
    function GetHintTwo: String;
    function GetHotTrack: Boolean;
    function GetHotTrackFontColor: TColor;
    function GetReadOnly: Boolean;
    function GetTransparentCenter: Boolean;
    procedure SetAlignment(Value: TLeftRight);
    procedure SetChecked(Value: Boolean);
    procedure SetCheckColor(Value: TColor);
    procedure SetCheckStyle(Value: TFoxCheckStyle);
    procedure SetCheckType(Value: TFoxCheckType);
    procedure SetCtl3D(Value: Boolean);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetHintTwo(Value: String);
    procedure SetHotTrack(Value: Boolean);
    procedure SetHotTrackFontColor(const Value: TColor);
    procedure SetReadOnly(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparentCenter(Value: Boolean);
    procedure SetVersion(const Value: String);
    procedure DrawFrame(R : TRect; C : TColor);
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TMessage); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
    procedure RepaintChildren;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TLeftRight read GetAlignment write SetAlignment;
    property Checked: Boolean read GetChecked write SetChecked;
    property CheckColor: TColor read GetCheckColor write SetCheckColor;
    property CheckStyle: TFoxCheckStyle read GetCheckStyle write SetCheckStyle;
    property CheckType: TFoxCheckType read GetCheckType write SetCheckType;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property FocusHighlight: Boolean read GetFocusHighlight write SetFocusHighlight;
    property HighlightColor: TColor read GetHighlightColor write SetHighlightColor;
    property HotTrack: Boolean read GetHotTrack write SetHotTrack;
    property HotTrackFontColor: TColor read GetHotTrackFontColor write SetHotTrackFontColor;
    property HintSecondLine: String read GetHintTwo write SetHintTwo;
    property OnMouseEnter: TOnMouseOverEvent read FOnMouseOver write FOnMouseOver;
    property OnMouseExit: TOnMouseOutEvent read FOnMouseOut write FOnMouseOut;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property TransparentCenter: Boolean read GetTransparentCenter write SetTransparentCenter;
    property Version: String read FVersion write SetVersion;
  end;

type
  TFoxPanel = class(TPanel)
  private
    FCtl3D: Boolean;
    FTransparent: Boolean;
    FTransparency: Byte;
    FVersion: String;
    procedure SetCtl3D(Value: Boolean);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparency(Value: Byte);
    procedure SetVersion(const Value: string);
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TWMERASEBKGND); message WM_ERASEBKGND;
  protected
    procedure Paint; override;
    procedure RepaintChildren;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Transparency: Byte read FTransparency write SetTransparency;
    property Version: String read FVersion write SetVersion;
  end;

procedure Register;

implementation

procedure TFoxCheckBox.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment:= Value;
    RecreateWnd;
  end;
end;

procedure TFoxCheckBox.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetBorder(Value: TFoxBorder);
begin
  if FBorder <> Value then
  begin
    FBorder:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor:= Value;
    RecreateWnd;
  end;
end;

procedure TFoxCheckBox.SetBorderVisible(Value: TFoxBorderVisible);
begin
  if FBorderVisible <> Value then
  begin
    FBorderVisible:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetChecked(Value: Boolean);
begin
  Fchecked:= Value;
  Invalidate;
end;

procedure TFoxCheckBox.SetCheckColor(Value: TColor);
begin
  if FCheckColor <> Value then
  begin
    FCheckColor:= Value;
    PickGlyph;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetCheckStyle(Value: TFoxCheckStyle);
begin
  if FCheckStyle <> Value then
  begin
    FCheckStyle:= Value;
    PickGlyph;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetCheckType(Value: TFoxCheckType);
begin
  if FCheckType <> Value then
  begin
    FCheckType:= Value;
    PickGlyph;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
  begin
    FCtl3D:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetFocusHighlight(Value: Boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxCheckBox.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
    FHighlightColor:= Value;
end;

procedure TFoxCheckBox.SetHintTwo(Value: String);
begin
  FHintTwo:= Value;
  if csDesigning in ComponentState then
    Exit;
  if hint <> '' then
    if FHintTwo <> '' then
      hint:= hint+#13+FHintTwo
    else
      hint:= hint;
end;

procedure TFoxCheckBox.SetHotTrack(Value: Boolean);
begin
  if FHotTrack <> Value then
    FHotTrack:= Value;
end;

procedure TFoxCheckBox.SetHotTrackFontColor(const Value: TColor);
begin
  if FHotTrackFontColor <> Value then
  begin
    FHotTrackFontColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetReadOnly(Value: Boolean);
begin
  FReadOnly:= Value;
end;

procedure TFoxCheckBox.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetTransparentCenter(Value: Boolean);
begin
  if FTransparentCenter <> Value then
  begin
    FTransparentCenter:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxCheckBox.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap:= Value;
    Invalidate;
  end;
end;

procedure TFoxCheckBox.DrawBorder;
begin
  case CurrentBorder of
    fbNone: Frame3d(Canvas, R, clBtnFace,clBtnFace,0);
    fbLowered: Frame3d(Canvas, R,clBtnShadow,clBtnHighlight,1);
    fbRaised: Frame3d(Canvas, R,clBtnHighlight,clBtnShadow,1);
    fbSimple: Frame3d(Canvas, R,FBorderColor,FBorderColor,1);
  end;

  // Frame3D shrinks R when it draws it so expand it back out
  if CurrentBorder > fbNone then
    OffsetRect(R,-1,0);
end;

procedure TFoxCheckBox.DrawCaption;
begin
  with Canvas do
  begin
    Brush.Style:= bsClear;

    if not Enabled then DrawCaptionDisabled;
    if FWordWrap then
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_WORDBREAK or DT_LEFT)
    else
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT);
    Font.Color:= Self.Font.Color;
  end;
end;

procedure TFoxCheckBox.DrawCaptionDisabled;
var
  ECaptRect: TRect;
begin
  with Canvas do
  begin
    DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT);
    font.Color:= clBtnHighlight;
    ECaptRect:=Rect(CaptRect.Left+1,CaptRect.top+1,CaptRect.Right+1,CaptRect.Bottom+1);
    DrawText(Handle, PChar(Caption), Length(Caption), ECaptRect, DT_LEFT);
    Font.color:= clBtnShadow;
  end;
end;

procedure TFoxCheckBox.DrawCheckBox;
var
  BL,BT,BR,BB,BM: Integer;
  R: TRect;
  CheckMarkRect: TRect;
begin
  with Canvas do
  begin
    BM:=14 div 2;
    BT:=(Height div 2)-BM;
    BB:=BT+13;
    BL:= 2;
    case FAlignment of
      taRightJustify: BL:=2;
      taLeftJustify: BL:=Width-16;
    end;
    BR:= 13+BL;
    R:= Rect(BL, BT, BR+1, BB+1);

    // Draw the checkbox
    if not FCtl3D then
    begin
      if Enabled then
        Pen.Color:= clWindowFrame
      else
        Pen.Color:= clBtnShadow;
      Rectangle(BL+1, BT+1, BR, BB);
      InflateRect(R, -2, -2);
    end
    else
    begin
      Frame3D(Canvas, R, clBtnShadow, clBtnHighlight, 1);
      Frame3D(Canvas, R, clWindowFrame, cl3DLight, 1);
    end;

    // Fill it
    if Enabled then
    begin
      if not FTransparentCenter then
      begin
        Brush.Color:= clWhite;
        FillRect(R);
      end;

      // Draw the checkmark
      if FChecked then
      begin
        TransColor:= FGlyph.Canvas.Pixels[0, FGlyph.Height - 1];
        CheckMarkRect:= Rect(0, 0, FGlyph.Width, FGlyph.Height);
        R:= Rect(BL-1,BT-1,BR+2,BB+2);
        Canvas.Brush.Style:= bsClear;
        {$IFDEF DELPHI_2}
          Canvas.BrushCopy(R, FGlyph, CheckMarkRect, clFuchsia);
        {$ELSE}
          Canvas.BrushCopy(R, FGlyph, CheckMarkRect, TransColor);
        {$ENDIF}
      end;
    end;
  end;
end;

procedure TFoxCheckBox.PickGlyph;
var
  gn : String;
begin
  gn:='CB_';
  case FCheckStyle of
    fsFlat: gn:= gn;
    fs3D: gn:= gn + '3D_';
    fs3DJumbo: gn:= gn + 'GLOSSY_';
  end;
  case FCheckType of
   fcCheck: gn:= gn + 'CHECK';
   fcCross: gn:= gn + 'CROSS';
   fcCircle: gn:= gn + 'CIRCLE';
  end;
  FGlyph.Handle:= LoadBitmap(hInstance, PChar(gn));
  ColorGlyph;
end;

procedure TFoxCheckBox.ColorGlyph;
var
  x,y,c: Integer;
  r,g,b: Byte;
  lr,lg,lb,hr,hg,hb : Byte;
begin
  c:= ColorToRGB(FCheckColor);
  r:= (c and $FF);
  g:= (c and $FF00) shr 8;
  b:= (c and $FF0000) shr 16;
  lr:= r shr 1;
  lg:= g shr 1;
  lb:= b shr 1;
  if r >= $80 then hr:= $FF else hr:= $FF - trunc(($FF-r)*0.25);
  if g >= $80 then hg:= $FF else hg:= $FF - trunc(($FF-g)*0.25);
  if b >= $80 then hb:= $FF else hb:= $FF - trunc(($FF-b)*0.25);

  for x:=0 to FGlyph.Width do
    for y:=0 to FGlyph.Height do
      // Change base color
      if FGlyph.Canvas.Pixels[x,y] = $0000FF then
        FGlyph.Canvas.Pixels[x,y]:= c
      else
        // Change shadow color
        if FGlyph.Canvas.Pixels[x,y] = $000080 then
          FGlyph.Canvas.Pixels[x,y]:= (lb shl 16) + (lg shl 8) + lr
        else
          // Change highlight color
          if FGlyph.Canvas.Pixels[x,y] = $00FFFF then
            FGlyph.Canvas.Pixels[x,y]:= (hb shl 16) + (hg shl 8) + hr;
end;

procedure TFoxCheckBox.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus and (not ReadOnly) then
    begin
      Click;
      Result:= 1;
      SetFocus;
      if FChecked then SetChecked(False) else SetChecked(True);
    end
    else
      inherited;
end;

procedure TFoxCheckBox.CMEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxCheckBox.CMMouseEnter(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= True;
    repaint;
  end;
  if Assigned(FOnMouseOver) then FOnMouseOver(Self);
end;

procedure TFoxCheckBox.CMMouseLeave(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= False;
    repaint;
  end;
  if not ButtonReleased then
  begin
    if FChecked then SetChecked(False) else SetChecked(True);
    ButtonReleased:= True;
  end;
  if Assigned(FOnMouseOut) then FOnMouseOut(Self);
end;

procedure TFoxCheckBox.CMTextChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxCheckBox.CMVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxCheckBox.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    Invalidate;
end;

procedure TFoxCheckBox.WMEraseBkgnd(var message: TMessage);
begin
  if FTransparent then
    message.Result:= 1
  else
    inherited;
end;

procedure TFoxCheckBox.WMLButtonDblClk (var Message: TWMLButtonDown);
begin
  inherited;
  Click;
end;

procedure TFoxCheckBox.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  Repaint;
end;

procedure TFoxCheckBox.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Repaint;
end;

procedure TFoxCheckBox.Click;
begin
  ButtonReleased:= True;
  if not FReadOnly then
    inherited;
end;

procedure TFoxCheckBox.paint;
var
  y: integer;
  NewWidth, TextSizeX, TextSizeY: Integer;
  TCaption: String;
begin
  // Figure out colors
  if (Focused) and (FFocusHighlight) then
    CurrentColor:= FHighlightColor
  else
    CurrentColor:= Color;
  if (IsHot) and (FHotTrack) then
    CurrentFontColor:= FHotTrackFontColor
  else
    CurrentFontColor:= Self.Font.Color;
  if not (Focused) and (FBorderVisible=bvWhenFocused) then
    CurrentBorder:= fbNone
  else
    CurrentBorder:= Border;

  if not (Focused) and not(ButtonReleased) then
    if FChecked then SetChecked(False) else SetChecked(True);

  R:= ClientRect;

  if FTransparent then
    DrawParentImage(Self, Self.Canvas)
  else
  begin
    Canvas.Brush.Color:= CurrentColor;
    Canvas.FillRect(R);
  end;

  if (IsHot) and (FHotTrack) then
    ApplyAlphaBlending(Canvas, clBtnHighlight, 50);

  DrawBorder;

  // Calculate size of caption rectangle
  TCaption:= Caption;
  if pos('&', TCaption)>0 then
    Delete(TCaption, pos('&', TCaption), 1);
  with Canvas do
  begin
    Font:= Self.Font;
    Font.color:= CurrentFontColor;
    if FWordWrap then
    begin
      CaptRect:= Rect(R.Left+20, R.Top, R.Right, R.Bottom);
      DrawText(Handle, PChar(TCaption), Length(TCaption), CaptRect, DT_WORDBREAK or DT_CALCRECT or DT_LEFT);
      TextSizeX:= CaptRect.Right - CaptRect.Left;
      TextSizeY:= CaptRect.Bottom - CaptRect.Top;
    end
    else
    begin
      TextSizeX:= TextWidth(TCaption);
      TextSizeY:= TextHeight(TCaption);
    end;
  end;
  y:= R.Top + ((R.Bottom-R.Top) - TextSizeY) div 2;
  case FAlignment of
    taRightJustify: CaptRect:= Rect(R.Left+20, y, R.Left+20+TextSizeX, y+TextSizeY);
    taLeftJustify: CaptRect:= Rect(R.Left+3, y, R.Left+3+TextSizeX, y+TextSizeY);
  end;

  // Draw focus rectangle
  if (Focused) then
  begin
    DrawFocusrect(Canvas.Handle,Rect(CaptRect.left-1,CaptRect.top-1,CaptRect.Right+1,CaptRect.bottom+1));
  end;

  DrawCaption;
  DrawCheckBox;
  if FAutoSize then
  begin
    NewWidth:= 22+Canvas.TextWidth(TCaption)+2;
    if FAlignment = taLeftJustify then Left:= left - (NewWidth - Width);
    Width:= NewWidth;
  end;
end;

procedure TFoxCheckBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_RETURN) or (Key = VK_SPACE) then
    if FChecked then SetChecked(False) else SetChecked(True);
  inherited KeyDown(Key, Shift);
end;

procedure TFoxCheckBox.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
Begin
  if Button <> mbleft then
    exit
  else
  begin
    if (not enabled) or (ReadOnly) then exit;
    SetFocus;
    if not FChecked then
    begin
      // We're not calling SetChecked yet because we may need to un-do
      FChecked:=True;
      Repaint;
    end
    else
    begin
      FChecked:=false;
      Repaint;
    end;
    ButtonReleased:= False;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

constructor TFoxCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle + [csReplicatable];
  SetBounds(0, 0, 97, 19);
  FAlignment:= taRightJustify;
  FAutoSize:= True;
  FBorder:= fbNone;
  FBorderColor:= clBlue;
  FBorderVisible:= bvAlways;
  FChecked:= False;
  FCheckStyle:= fs3D;
  FCheckType:= fcCross;
  FCheckColor:= clRed;
  FCtl3D:= True;
  FFocusHighlight:=False;
  FHighlightColor:= clLime;
  FHotTrack:= True;
  FHotTrackFontColor:= clBlue;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  FWordWrap:= False;
  ButtonReleased:= True;
  TabStop:=True;
  FGlyph:= TBitmap.Create;
  PickGlyph;
end;

destructor TFoxCheckBox.Destroy;
begin
  inherited Destroy;
  FGlyph.Free;
end;

// ------------------------------------------------------------------------

procedure TFoxGroupCheck.Click;
begin
  inherited;
  With Parent as TFoxGroupBoxWithCheck do
  begin
    SetChecked(Self.Checked);
    If Assigned(OnClick) then OnClick(Self);  // Trigger the groupbox's OnClick event
  end;
end;

constructor TFoxGroupCheck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  // Position self at top of parent FoxGroupBox
  Top:= 0;
  if not UseRightToLeftAlignment then
    Left:= 8
  else
    Left:= Parent.Width - Self.Width - 8;
end;

destructor TFoxGroupCheck.Destroy;
begin
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure TFoxRadioButton.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment:= value;
    RecreateWnd;
  end;
end;

procedure TFoxRadioButton.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetBorder(Value: TFoxBorder);
begin
  if FBorder <> Value then
  begin
    FBorder:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor:= value;
    RecreateWnd;
  end;
end;

procedure TFoxRadioButton.SetBorderVisible(Value: TFoxBorderVisible);
begin
  if FBorderVisible <> Value then
  begin
    FBorderVisible:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetChecked(Value: Boolean);
begin
  PrevCheckState:= FChecked;
  Fchecked:= Value;
  if FChecked then
    TurnSiblingsOff;
  Invalidate;
end;

procedure TFoxRadioButton.SetCheckColor(Value: TColor);
begin
  if FCheckColor <> Value then
  begin
    FCheckColor:= Value;
    PickGlyph;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetCheckStyle(Value: TFoxCheckStyle);
begin
  if FCheckStyle <> Value then
  begin
    FCheckStyle:= Value;
    PickGlyph;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
  begin
    FCtl3D:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetFocusHighlight(Value: Boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxRadioButton.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
    FHighlightColor:= Value;
end;

procedure TFoxRadioButton.SetHintTwo(Value: String);
begin
  FHintTwo:= Value;
  if csDesigning in ComponentState then
    Exit;
  if hint <> '' then
    if FHintTwo <> '' then
      hint:= hint+#13+FHintTwo
    else
      hint:= hint;
end;

procedure TFoxRadioButton.SetHotTrack(Value: Boolean);
begin
  if FHotTrack <> Value then
    FHotTrack:= Value;
end;

procedure TFoxRadioButton.SetHotTrackFontColor(const Value: TColor);
begin
  if FHotTrackFontColor <> Value then
  begin
    FHotTrackFontColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetRaised(Value: Boolean);
begin
  if FRaised <> Value then
  begin
    FRaised:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetReadOnly(Value: Boolean);
begin
  FReadOnly:= Value;
end;

procedure TFoxRadioButton.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:=Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetTransparentCenter(Value: Boolean);
begin
  if FTransparentCenter <> Value then
  begin
    FTransparentCenter:=value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap:= Value;
    Invalidate;
  end;
end;

procedure TFoxRadioButton.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxRadioButton.TurnSiblingsOff;
var
  i:Integer;
  Sibling: TFoxRadioButton;
begin
  if Parent <> nil then
    for i:=0 to Parent.ControlCount-1 do
      if Parent.Controls[i] is TFoxRadioButton then
      begin
        Sibling:= TFoxRadioButton(Parent.Controls[i]);
        if (Sibling <> Self) and (Sibling.GroupIndex=GroupIndex) then
           Sibling.SetChecked(False);
      end;
end;

procedure TFoxRadioButton.DrawBorder;
begin
  case CurrentBorder of
    fbNone: Frame3d(canvas, R, clBtnFace,clBtnFace,0);
    fbLowered: Frame3d(canvas, R,clBtnShadow,clBtnHighlight,1);
    fbRaised: Frame3d(canvas, R,clBtnHighlight,clBtnShadow,1);
    fbSimple: Frame3d(canvas, R,FBorderColor,FBorderColor,1);
  end;

  // Frame3D shrinks R when it draws it so expand it back out
  if CurrentBorder > fbNone then
    OffsetRect(R,-1,0);
end;

procedure TFoxRadioButton.DrawCaption;
begin
  with Canvas do
  begin
    Brush.Style:= bsClear;

    if not Enabled then DrawCaptionDisabled;
    if FWordWrap then
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_WORDBREAK or DT_LEFT)
    else
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT);
    Font.Color:= Self.Font.Color;
  end;
end;

procedure TFoxRadioButton.DrawCaptionDisabled;
var
  ECaptRect: TRect;
begin
  with canvas do
  begin
    DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT);
    font.Color:= clBtnHighlight;
    ECaptRect:= Rect(CaptRect.Left+1,CaptRect.top+1,CaptRect.Right+1,CaptRect.Bottom+1);
    DrawText(Handle, PChar(Caption), Length(Caption), ECaptRect, DT_LEFT);
    font.color:= clBtnShadow;
  end;
end;

procedure TFoxRadioButton.DrawButton;
var
  BL,BT,BR,BB,BM:Integer;
  ButtonRect: TRect;
  TransColor: TColor;
begin
  with Canvas do
  begin
    BM:=12 div 2;
    BT:=(Height div 2)-BM;
    BB:=BT+13;
    BL:= 3;
    case FAlignment of
      taRightJustify: BL:=3;
      taLeftJustify: BL:=Width-15;
    end;
    BR:= 13+BL;

    if not FCtl3D then
    begin
      if Enabled then
        Pen.Color:= clWindowFrame
      else
        Pen.COlor:= clBtnShadow;
      Ellipse(BL,BT,BR,BB);
    end
    else
    begin
      if (not FTransparentCenter) and (Enabled) then
        Pen.Width:= 2; // To fill in any holes between arcs
      if not (FRaised)
       or (FRaised and Checked and Enabled) then
      begin
        Pen.Color:= clbtnShadow;
        Arc(BL,BT,BR,BB,BR,BT,BL,BB);
        Pen.Color:= cl3DLight;
        Arc(BL,BT,BR,BB,BL,BB,BR,BT);
      end
      else
      begin
        Pen.Color:= Cl3DLight;
        Arc(BL,BT,BR,BB,BR,BT,BL,BB);
        Pen.Color:= ClBtnShadow;
        Arc(BL,BT,BR,BB,BL,BB,BR,BT);
      end;
      Pen.Width:= 1;
    end;

    if Enabled then
    begin
      if not FTransparentCenter then
      begin
        Pen.Color:= clWhite;
        Brush.Color:= clWhite;
        Ellipse(BL+1,Bt+1,BR-1,BB-1);
      end;

      if FChecked then
      begin
        TransColor:= FGlyph.Canvas.Pixels[0, FGlyph.Height - 1];
        ButtonRect:= Rect(0, 0, FGlyph.Width, FGlyph.Height);
        if FCheckStyle = fs3DJumbo then
          R:= Rect(BL+2,BT+2,BR-2,BB-2)
        else
          R:= Rect(BL+3,BT+3,BR-3,BB-3);
        Canvas.Brush.Style:= bsClear;
        {$IFDEF DELPHI_2}
          Canvas.BrushCopy(R, FGlyph, ButtonRect, clFuchsia);
        {$ELSE}
          Canvas.BrushCopy(R, FGlyph, ButtonRect, TransColor);
        {$ENDIF}
      end;
    end;
  end;
end;

procedure TFoxRadioButton.PickGlyph;
begin
  case FCheckStyle of
    fsFlat: FGlyph.Handle:= LoadBitmap(hInstance, 'RB_CIRCLE');
    fs3D: FGlyph.Handle:= LoadBitmap(hInstance, 'RB_3D_CIRCLE');
    fs3DJumbo: FGlyph.Handle:= LoadBitmap(hInstance, 'RB_GLOSSY_CIRCLE');
  end;
  ColorGlyph;
end;

procedure TFoxRadioButton.ColorGlyph;
var
  x,y,c: Integer;
  r,g,b: Byte;
  lr,lg,lb,hr,hg,hb : Byte;
begin
  c:= ColorToRGB(FCheckColor);
  r:= (c and $FF);
  g:= (c and $FF00) shr 8;
  b:= (c and $FF0000) shr 16;
  lr:= r shr 1;
  lg:= g shr 1;
  lb:= b shr 1;
  if r >= $80 then hr:= $FF else hr:= $FF - trunc(($FF-r)*0.25);
  if g >= $80 then hg:= $FF else hg:= $FF - trunc(($FF-g)*0.25);
  if b >= $80 then hb:= $FF else hb:= $FF - trunc(($FF-b)*0.25);

  for x:=0 to FGlyph.Width do
    for y:=0 to FGlyph.Height do
      // Change base color
      if FGlyph.Canvas.Pixels[x,y] = $0000FF then
        FGlyph.Canvas.Pixels[x,y]:= c
      else
        // Change shadow color
        if FGlyph.Canvas.Pixels[x,y] = $000080 then
          FGlyph.Canvas.Pixels[x,y]:= (lb shl 16) + (lg shl 8) + lr
        else
          // Change highlight color
          if FGlyph.Canvas.Pixels[x,y] = $00FFFF then
            FGlyph.Canvas.Pixels[x,y]:= (hb shl 16) + (hg shl 8) + hr;
end;

procedure TFoxRadioButton.CMDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus and (not ReadOnly) then
    begin
      Click;
      Result:= 1;
      SetFocus;
      if FChecked then SetChecked(False) else SetChecked(True);
    end
    else
      inherited;
end;

procedure TFoxRadioButton.CMEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxRadioButton.CMMouseEnter(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= True;
    repaint;
  end;
  if Assigned(FOnMouseOver) then FOnMouseOver(Self);
end;

procedure TFoxRadioButton.CMMouseLeave(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= False;
    repaint;
  end;
  if not ButtonReleased then
  begin
    if (FChecked) and (not PrevCheckState) then
    begin
      SetChecked(False);
      PrevCheckState:= False;
    end;
    ButtonReleased:= True;
  end;
  if Assigned(FOnMouseOut) then FOnMouseOut(Self);
end;

procedure TFoxRadioButton.CMTextChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxRadioButton.CMVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxRadioButton.CNCommand(var Message: TWMCommand);
begin
  case Message.NotifyCode of
    BN_CLICKED: SetChecked(True);
    BN_DOUBLECLICKED: DblClick;
  end;
end;

procedure TFoxRadioButton.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    Invalidate;
end;

procedure TFoxRadioButton.WMEraseBkgnd(var message: TMessage);
begin
  if FTransparent then
    message.Result:= 1
  else
    inherited;
end;

procedure TFoxRadioButton.WMLButtonDblClk (var Message: TWMLButtonDown);
begin
  inherited;
  Click;
end;

procedure TFoxRadioButton.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  Repaint;
end;

procedure TFoxRadioButton.WMKillFocus(var Message: TWMKillFocus);
begin
  inherited;
  Repaint;
end;

procedure TFoxRadioButton.Click;
begin
  ButtonReleased:= True;
  if not FReadOnly then
  begin
    SetChecked(True);
    inherited;
  end;
end;

procedure TFoxRadioButton.paint;
var
  y: integer;
  NewWidth, TextSizeX, TextSizeY: Integer;
  TCaption: String;
begin
  // Figure out colors
  if (Focused) and (FFocusHighlight) then
    CurrentColor:= FHighlightColor
  else
    CurrentColor:= Color;
  if not (Focused) and (FBorderVisible=bvWhenFocused) then
    CurrentBorder:= fbNone
  else
    CurrentBorder:= Border;
  if (IsHot) and (FHotTrack) then
    CurrentFontColor:= FHotTrackFontColor
  else
    CurrentFontColor:= Self.Font.Color;

  R:= ClientRect;

  if FTransparent then
    DrawParentImage(Self, Self.Canvas)
  else
  begin
    Canvas.Brush.Color:= CurrentColor;
    Canvas.FillRect(R);
  end;

  if (IsHot) and (FHotTrack) then
    ApplyAlphaBlending(Canvas, clBtnHighlight, 50);

  DrawBorder;

  // Calculate size of caption rectangle
  TCaption:= Caption;
  if pos('&', TCaption)>0 then
    Delete(TCaption, pos('&', TCaption), 1);
  with Canvas do
  begin
    Font:= Self.Font;
    Font.Color:= CurrentFontColor;
    if FWordWrap then
    begin
      CaptRect:= Rect(R.Left+20, R.Top, R.Right, R.Bottom);
      DrawText(Handle, PChar(TCaption), Length(TCaption), CaptRect, DT_WORDBREAK or DT_CALCRECT or DT_LEFT);
      TextSizeX:= CaptRect.Right - CaptRect.Left;
      TextSizeY:= CaptRect.Bottom - CaptRect.Top;
    end
    else
    begin
      TextSizeX:= TextWidth(TCaption);
      TextSizeY:= TextHeight(TCaption);
    end;
  end;
  y:= R.Top + ((R.Bottom-R.Top) - TextSizeY) div 2;
  case FAlignment of
    taRightJustify: CaptRect:= Rect(R.Left+20, y, R.Left+20+TextSizeX, y+TextSizeY);
    taLeftJustify: CaptRect:= Rect(R.Left+3, y, R.Left+3+TextSizeX, y+TextSizeY);
  end;

  // Draw focus rectangle
  if (Focused) then
  begin
    DrawFocusrect(Canvas.Handle,Rect(CaptRect.left-1,CaptRect.top-1,CaptRect.Right+1,CaptRect.bottom+1));
  end;

  DrawCaption;
  DrawButton;
  if FAutoSize then
  begin
    NewWidth:= 22+Canvas.TextWidth(TCaption)+2;
    if FAlignment = taLeftJustify then Left:= left - (NewWidth - Width);
    Width:= NewWidth;
  end;
end;

procedure TFoxRadioButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_RETURN) or (Key = VK_SPACE) then
  begin
    SetChecked(True);
    PrevCheckState:= True;
  end;
  inherited KeyDown(Key, Shift);
end;

procedure TFoxRadioButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
Begin
  if Button <> mbleft then
    exit
  else
  begin
    if (not enabled) or (ReadOnly) then exit;
    SetFocus;
    if not FChecked then
    begin
      // We're not calling SetChecked yet because we may need to un-do
      PrevCheckState:= FChecked;
      FChecked:= True;
      Repaint;
    end;
    ButtonReleased:= False;
  end;
  inherited MouseDown(Button, Shift, X, Y);
end;

constructor TFoxRadioButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle + [csReplicatable];
  SetBounds(0, 0, 97, 19);
  FAlignment:= taRightJustify;
  FAutoSize:= True;
  FBorder:= fbNone;
  FBorderColor:= clBlue;
  FBorderVisible:= bvAlways;
  FCheckStyle:= fs3D;
  FChecked:= False;
  FCheckColor:= clBlack;
  FCtl3D:= True;
  FFocusHighlight:=False;
  FHighlightColor:= clLime;
  FHotTrack:= True;
  FHotTrackFontColor:= clBlue;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  FWordWrap:= False;
  ButtonReleased:= True;
  TabStop:=True;
  FGlyph:= TBitmap.Create;
  PickGlyph;
  PrevCheckState:= False;
end;

destructor TFoxRadioButton.Destroy;
begin
  inherited Destroy;
  FGlyph.Free;
end;

// ------------------------------------------------------------------------

procedure TFoxGroupButton.CNCommand(var Message: TWMCommand);
begin
  if not FInClick then
  begin
    FInClick:= True;
    try
      if ((Message.NotifyCode = BN_CLICKED) or
        (Message.NotifyCode = BN_DOUBLECLICKED)) and
        TFoxRadioGroup(Parent).CanModify then
        inherited;
    except
      Application.HandleException(Self);
    end;
    FInClick:= False;
  end;
end;

procedure TFoxGroupButton.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  TFoxRadioGroup(Parent).KeyPress(Key);
  if (Key = #8) or (Key = ' ') then
  begin
    if not TFoxRadioGroup(Parent).CanModify then Key:= #0;
  end;
end;

procedure TFoxGroupButton.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited KeyDown(Key, Shift);
  TFoxRadioGroup(Parent).KeyDown(Key, Shift);
end;

constructor TFoxGroupButton.InternalCreate(RadioGroup: TFoxRadioGroup);
begin
  inherited Create(RadioGroup);
  RadioGroup.FButtons.Add(Self);
  Visible:= False;
  Enabled:= RadioGroup.Enabled;
  ParentShowHint:= False;
  OnClick:= RadioGroup.ButtonClick;
  Parent:= RadioGroup;
end;

destructor TFoxGroupButton.Destroy;
begin
  TFoxRadioGroup(Owner).FButtons.Remove(Self);
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure TFoxRadioGroup.FlipChildren(AllLevels: Boolean);
begin
  { The radio buttons are flipped using BiDiMode }
end;

procedure TFoxRadioGroup.SetBorder(Value: TFoxBorder);
begin
  if FBorder <> Value then
  begin
    FBorder:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor:= value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetAlignment(Value: TLeftRight);
begin
  if FAlignment <> Value then
  begin
    FAlignment:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetBorderVisible(Value: TFoxBorderVisible);
begin
  if FBorderVisible <> Value then
  begin
    FBorderVisible:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetButtonCount(Value: Integer);
begin
  while FButtons.Count < Value do TFoxGroupButton.InternalCreate(Self);
  while FButtons.Count > Value do TFoxGroupButton(FButtons.Last).Free;
end;

procedure TFoxRadioGroup.SetCheckColor(Value: TColor);
begin
  if FCheckColor <> Value then
  begin
    FCheckColor:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetCheckStyle(Value: TFoxCheckStyle);
begin
  if FCheckStyle <> Value then
  begin
    FCheckStyle:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetColumns(Value: Integer);
begin
  if Value < 1 then Value:= 1;
  if Value > 16 then Value:= 16;
  if FColumns <> Value then
  begin
    FColumns:= Value;
    ArrangeButtons;
    Invalidate;
  end;
end;

procedure TFoxRadioGroup.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
  begin
    FCtl3D:= Value;
    ArrangeButtons;
    Invalidate;
  end;
end;

procedure TFoxRadioGroup.SetFocusHighlight(Value: Boolean);
begin
  if FFocusHighlight <> Value then
  begin
    FFocusHighlight:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
  begin
    FHighlightColor:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetHotTrack(Value: Boolean);
begin
  if FHotTrack <> Value then
  begin
    FHotTrack:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetHotTrackFontColor(const Value: TColor);
begin
  if FHotTrackFontColor <> Value then
  begin
    FHotTrackFontColor:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetItemIndex(Value: Integer);
begin
  if FReading then FItemIndex:= Value else
  begin
    if Value < -1 then Value:= -1;
    if Value >= FButtons.Count then Value:= FButtons.Count - 1;
    if FItemIndex <> Value then
    begin
      if FItemIndex >= 0 then
        TFoxGroupButton(FButtons[FItemIndex]).Checked:= False;
      FItemIndex:= Value;
      if FItemIndex >= 0 then
        TFoxGroupButton(FButtons[FItemIndex]).Checked:= True;
    end;
  end;
end;

procedure TFoxRadioGroup.SetItems(Value: TStrings);
begin
  FItems.Assign(Value);
end;

procedure TFoxRadioGroup.SetRaised(Value: Boolean);
begin
  if FRaised <> Value then
  begin
    FRaised:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetReadOnly(Value: Boolean);
begin
  if FReadOnly <> Value then
  begin
    FReadOnly:= Value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    ArrangeButtons;
    Invalidate;
  end;
end;

procedure TFoxRadioGroup.SetTransparentCenter(Value: Boolean);
begin
  if FTransparentCenter <> Value then
  begin
    FTransparentCenter:=value;
    ArrangeButtons;
  end;
end;

procedure TFoxRadioGroup.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxRadioGroup.DrawFrame(R : TRect; C : TColor);
var offset : Integer;
begin
  if C = clBtnHighlight then offset:= 6 else offset:= 7;
  with Canvas do
  begin
    Pen.Color:= C;
    MoveTo(R.Left + offset, R.Top); // 2-pixel space from caption
    LineTo(R.Left, R.Top);
    LineTo(R.Left, R.Bottom);
    LineTo(R.Right, R.Bottom);
    LineTo(R.Right, R.Top);
    LineTo(R.Left + offset + TextWidth(Caption), R.Top); // 3-pixel space from caption
  end;
end;

procedure TFoxRadioGroup.ArrangeButtons;
var
  ButtonsPerCol, ButtonWidth, ButtonHeight, TopMargin, I: Integer;
  DC: HDC;
  SaveFont: HFont;
  Metrics: TTextMetric;
  DeferHandle: THandle;
  ALeft: Integer;
begin
  if (FButtons.Count <> 0) and not FReading then
  begin
    DC:= GetDC(0);
    SaveFont:= SelectObject(DC, Font.Handle);
    GetTextMetrics(DC, Metrics);
    SelectObject(DC, SaveFont);
    ReleaseDC(0, DC);
    ButtonsPerCol:= (FButtons.Count + FColumns - 1) div FColumns;
    ButtonWidth:= (Width - 15) div FColumns;
    I:= Height - Metrics.tmHeight - 5;
    ButtonHeight:= I div ButtonsPerCol;
    TopMargin:= Metrics.tmHeight + 1 + (I mod ButtonsPerCol) div 2;
    DeferHandle:= BeginDeferWindowPos(FButtons.Count);
    try
      for I:= 0 to FButtons.Count - 1 do
        with TFoxGroupButton(FButtons[I]) do
        begin
          BiDiMode:= Self.BiDiMode;
          ALeft:= (I div ButtonsPerCol) * ButtonWidth + 8;
          if UseRightToLeftAlignment then
            ALeft:= Self.ClientWidth - ALeft - ButtonWidth;
          DeferHandle:= DeferWindowPos(DeferHandle, Handle, 0,
            ALeft,
            (I mod ButtonsPerCol) * ButtonHeight + TopMargin,
            ButtonWidth, ButtonHeight,
            SWP_NOZORDER or SWP_NOACTIVATE);
          Visible:= True;
          // Copy parent visual settings
          Alignment:= Self.Alignment;
          AutoSize:= Self.AutoSize;
          Border:= Self.Border;
          BorderColor:= Self.BorderColor;
          BorderVisible:= Self.BorderVisible;
          CheckStyle:= Self.CheckStyle;
          CheckColor:= Self.CheckColor;
          Ctl3D:= Self.Ctl3D;
          FocusHighlight:= Self.FocusHighlight;
          HighlightColor:= Self.HighlightColor;
          HotTrack:= Self.HotTrack;
          HotTrackFontColor:= Self.HotTrackFontColor;
          Raised:= Self.Raised;
          ReadOnly:= Self.ReadOnly;
          Transparent:= Self.Transparent;
          TransparentCenter:= Self.TransparentCenter;
        end;
    finally
      EndDeferWindowPos(DeferHandle);
    end;
  end;
end;

procedure TFoxRadioGroup.ButtonClick(Sender: TObject);
begin
  if not FUpdating then
  begin
    FItemIndex:= FButtons.IndexOf(Sender);
    Changed;
    Click;
  end;
end;

procedure TFoxRadioGroup.ItemsChange(Sender: TObject);
begin
  if not FReading then
  begin
    if FItemIndex >= FItems.Count then FItemIndex:= FItems.Count - 1;
    UpdateButtons;
  end;
end;

procedure TFoxRadioGroup.Loaded;
begin
  inherited Loaded;
  ArrangeButtons;
end;

procedure TFoxRadioGroup.paint;
var
  R: TRect;
  Flags: Longint;
begin
  if FTransparent then
    DrawParentImage(Self, Self.Canvas);
  with Canvas do
  begin
    Font:= Self.Font;
    R:= Rect(0, TextHeight(Caption) div 2 - 1, Width-1, Height-1);
    if Ctl3D then
    begin
      Inc(R.Left);
      Inc(R.Top);
      Brush.Color:= clBtnHighlight;
      DrawFrame(R, Brush.Color);
      OffsetRect(R, -1, -1);
      Brush.Color:= clBtnShadow;
    end
    else
      Brush.Color:= clWindowFrame;
    DrawFrame(R, Brush.Color);
    if Caption <> '' then
    begin
      if not UseRightToLeftAlignment then
        R:= Rect(8, 0, 0, TextHeight(Caption))
      else
        R:= Rect(R.Right - Canvas.TextWidth(Text) - 8, 0, 0, TextHeight(Caption));
      Flags:= DrawTextBiDiModeFlags(DT_SINGLELINE);
      DrawText(Handle, PChar(Caption), Length(Caption), R, Flags or DT_CALCRECT);
      if not FTransparent then
        Brush.Color:= Color
      else
        Brush.Style:= bsClear;
      DrawText(Handle, PChar(Caption), Length(Caption), R, Flags);
    end;
  end;
end;

procedure TFoxRadioGroup.ReadState(Reader: TReader);
begin
  FReading:= True;
  inherited ReadState(Reader);
  FReading:= False;
  UpdateButtons;
end;

procedure TFoxRadioGroup.UpdateButtons;
var
  I: Integer;
begin
  SetButtonCount(FItems.Count);
  for I:= 0 to FButtons.Count - 1 do
    TFoxGroupButton(FButtons[I]).Caption:= FItems[I];
  if FItemIndex >= 0 then
  begin
    FUpdating:= True;
    TFoxGroupButton(FButtons[FItemIndex]).Checked:= True;
    FUpdating:= False;
  end;
  ArrangeButtons;
  Invalidate;
end;

procedure TFoxRadioGroup.CMEnabledChanged(var Message: TMessage);
var
  I: Integer;
begin
  inherited;
  for I:= 0 to FButtons.Count - 1 do
    TFoxGroupButton(FButtons[I]).Enabled:= Enabled;
end;

procedure TFoxRadioGroup.CMFontChanged(var Message: TMessage);
begin
  inherited;
  ArrangeButtons;
end;

procedure TFoxRadioGroup.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    Invalidate;
end;

procedure TFoxRadioGroup.WMEraseBkgnd(var message: TMessage);
begin
  if FTransparent then
    message.Result:= 1
  else
    inherited;
end;

procedure TFoxRadioGroup.WMSize(var Message: TWMSize);
begin
  inherited;
  ArrangeButtons;
end;

function TFoxRadioGroup.CanModify: Boolean;
begin
  Result:= True;
end;

procedure TFoxRadioGroup.GetChildren(Proc: TGetChildProc; Root: TComponent);
begin
end;

constructor TFoxRadioGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= [csSetCaption, csDoubleClicks];
  FAlignment:= taRightJustify;
  FBorder:= fbNone;
  FBorderColor:= clBlue;
  FBorderVisible:= bvAlways;
  FButtons:= TList.Create;
  FCheckStyle:= fs3D;
  FColumns:= 1;
  FCtl3D:= True;
  FFocusHighlight:=False;
  FHighlightColor:= clLime;
  FHotTrack:= True;
  FHotTrackFontColor:= clBlue;
  FItems:= TStringList.Create;
  TStringList(FItems).OnChange:= ItemsChange;
  FItemIndex:= -1;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
end;

destructor TFoxRadioGroup.Destroy;
begin
  SetButtonCount(0);
  TStringList(FItems).OnChange:= nil;
  FItems.Free;
  FButtons.Free;
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure TFoxGroupBox.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
    FCtl3D:= Value;
  Repaint;
end;

procedure TFoxGroupBox.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
    RepaintChildren;
  end;
end;

procedure TFoxGroupBox.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxGroupBox.DrawFrame(R : TRect; C : TColor);
var offset : Integer;
begin
  if C = clBtnHighlight then offset:= 6 else offset:= 7;
  with Canvas do
  begin
    Pen.Color:= C;
    MoveTo(R.Left + offset, R.Top); // 2-pixel space from caption
    LineTo(R.Left, R.Top);
    LineTo(R.Left, R.Bottom);
    LineTo(R.Right, R.Bottom);
    LineTo(R.Right, R.Top);
    LineTo(R.Left + offset + TextWidth(Caption), R.Top); // 3-pixel space from caption
  end;
end;

procedure TFoxGroupBox.paint;
var
  R: TRect;
  Flags: Longint;
begin
  if FTransparent then
    DrawParentImage(Self, Self.Canvas);
  with Canvas do
  begin
    Font:= Self.Font;
    R:= Rect(0, TextHeight(Caption) div 2 - 1, Width-1, Height-1);
    if Ctl3D then
    begin
      Inc(R.Left);
      Inc(R.Top);
      Brush.Color:= clBtnHighlight;
      DrawFrame(R, Brush.Color);
      OffsetRect(R, -1, -1);
      Brush.Color:= clBtnShadow;
    end else
      Brush.Color:= clWindowFrame;
    DrawFrame(R, Brush.Color);
    if Caption <> '' then
    begin
      if not UseRightToLeftAlignment then
        R:= Rect(8, 0, 0, TextHeight(Caption))
      else
        R:= Rect(R.Right - Canvas.TextWidth(Caption) - 8, 0, 0, TextHeight(Caption));
      Flags:= DrawTextBiDiModeFlags(DT_SINGLELINE);
      DrawText(Handle, PChar(Caption), Length(Caption), R, Flags or DT_CALCRECT);
      if not FTransparent then
        Brush.Color:= Color
      else
        Brush.Style:= bsClear;
      DrawText(Handle, PChar(Caption), Length(Caption), R, Flags);
    end;
  end;
end;

procedure TFoxGroupBox.RepaintChildren;
var
  I : Integer;
begin
  if ControlCount > 0 then
  begin
    for I:=0 to ControlCount-1 do
      Controls[i].Invalidate;
  end;
end;

procedure TFoxGroupBox.CMEnabledChanged(var Message: TWmNoParams);
var
  I: Integer;
begin
  inherited;
  if ControlCount > 0 then
    for I:=0 to ControlCount-1 do
      Controls[i].Enabled:= Enabled;
end;

procedure TFoxGroupBox.WMMove(var message:TMessage);
begin
  inherited;
  Invalidate;
  RepaintChildren;
end;

procedure TFoxGroupBox.WMEraseBkgnd(var message: TMessage);
begin
  if FTransparent then
    message.Result:= 1
  else
    inherited;
end;

constructor TFoxGroupBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCtl3D:= True;
  ControlStyle:= ControlStyle + [csAcceptsControls, csReplicatable];
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  TabStop:=True;
end;

destructor TFoxGroupBox.Destroy;
begin
  inherited Destroy;
end;

// ------------------------------------------------------------------------

function TFoxGroupBoxWithCheck.GetAlignment: TLeftRight;
begin
  Result:= FCheckBox.Alignment;
end;

function TFoxGroupBoxWithCheck.GetChecked: Boolean;
begin
  Result:= FCheckBox.Checked;
end;

function TFoxGroupBoxWithCheck.GetCheckColor: TColor;
begin
  Result:= FCheckBox.CheckColor;
end;

function TFoxGroupBoxWithCheck.GetCheckStyle: TFoxCheckStyle;
begin
  Result:= FCheckBox.CheckStyle;
end;

function TFoxGroupBoxWithCheck.GetCheckType: TFoxCheckType;
begin
  Result:= FCheckBox.CheckType;
end;

function TFoxGroupBoxWithCheck.GetFocusHighlight: Boolean;
begin
  Result:= FCheckBox.FocusHighlight;
end;

function TFoxGroupBoxWithCheck.GetHighlightColor: TColor;
begin
  Result:= FCheckBox.HighlightColor;
end;

function TFoxGroupBoxWithCheck.GetHintTwo: String;
begin
  Result:= FCheckBox.HintSecondLine;
end;

function TFoxGroupBoxWithCheck.GetHotTrack: Boolean;
begin
  Result:= FCheckBox.HotTrack;
end;

function TFoxGroupBoxWithCheck.GetHotTrackFontColor: TColor;
begin
  Result:= FCheckBox.HotTrackFontColor;
end;

function TFoxGroupBoxWithCheck.GetReadOnly: Boolean;
begin
  Result:= FCheckBox.ReadOnly;
end;

function TFoxGroupBoxWithCheck.GetTransparentCenter: Boolean;
begin
  Result:= FCheckBox.TransparentCenter;
end;

procedure TFoxGroupBoxWithCheck.SetAlignment(Value: TLeftRight);
begin
  FCheckBox.Alignment:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetChecked(Value: Boolean);
var
  I: Integer;
begin
  FCheckBox.Checked:= Value;
  if ControlCount > 0 then
    for I:=0 to ControlCount-1 do
      if Controls[i] <> FCheckBox then
        Controls[i].Enabled:= (Enabled and Value);
end;

procedure TFoxGroupBoxWithCheck.SetCheckColor(Value: TColor);
begin
  FCheckBox.CheckColor:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetCheckStyle(Value: TFoxCheckStyle);
begin
  FCheckBox.CheckStyle:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetCheckType(Value: TFoxCheckType);
begin
  FCheckBox.CheckType:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
    FCtl3D:= Value;
  Repaint;
end;

procedure TFoxGroupBoxWithCheck.SetFocusHighlight(Value: Boolean);
begin
  FCheckBox.FocusHighlight:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetHighlightColor(Value: TColor);
begin
  FCheckBox.HighlightColor:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetHintTwo(Value: String);
begin
  FCheckBox.HintSecondLine:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetHotTrack(Value: Boolean);
begin
  FCheckBox.HotTrack:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetHotTrackFontColor(const Value: TColor);
begin
  FCheckBox.HotTrackFontColor:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetReadOnly(Value: Boolean);
begin
  FCheckBox.ReadOnly:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    FCheckBox.Transparent:= Value;
    Invalidate;
  end;
end;

procedure TFoxGroupBoxWithCheck.SetTransparentCenter(Value: Boolean);
begin
  FCheckBox.TransparentCenter:= Value;
end;

procedure TFoxGroupBoxWithCheck.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxGroupBoxWithCheck.DrawFrame(R : TRect; C : TColor);
var offset : Integer;
begin
  if C = clBtnHighlight then offset:= 6 else offset:= 7;
  with Canvas do
  begin
    Pen.Color:= C;
    MoveTo(R.Left + offset, R.Top); // 2-pixel space from caption
    LineTo(R.Left, R.Top);
    LineTo(R.Left, R.Bottom);
    LineTo(R.Right, R.Bottom);
    LineTo(R.Right, R.Top);
    LineTo(R.Left + offset + FCheckBox.Width, R.Top); // 3-pixel space from caption
  end;
end;

procedure TFoxGroupBoxWithCheck.paint;
var
  R: TRect;
begin
  if FTransparent then
    DrawParentImage(Self, Self.Canvas);
  with Canvas do
  begin
    Font:= Self.Font;
    R:= Rect(0, (TextHeight(Caption) + 3) div 2 - 1, Width-1, Height-1);
    if Ctl3D then
    begin
      Inc(R.Left);
      Inc(R.Top);
      Brush.Color:= clBtnHighlight;
      DrawFrame(R, Brush.Color);
      OffsetRect(R, -1, -1);
      Brush.Color:= clBtnShadow;
    end
    else
      Brush.Color:= clWindowFrame;
    DrawFrame(R, Brush.Color);
    FCheckBox.Caption:= Self.Caption;
    FCheckBox.Ctl3D:= Self.Ctl3D;
  end;
end;

procedure TFoxGroupBoxWithCheck.RepaintChildren;
var
  I : Integer;
begin
  if ControlCount > 0 then
  begin
    for I:=0 to ControlCount-1 do
      Controls[i].Invalidate;
  end;
end;

procedure TFoxGroupBoxWithCheck.CMEnabledChanged(var Message: TWmNoParams);
var
  I: Integer;
begin
  inherited;
  FCheckBox.Enabled:= Enabled;  // Our internal checkbox
  if ControlCount > 0 then
    for I:=0 to ControlCount-1 do
      if Controls[i] <> FCheckBox then
        Controls[i].Enabled:= (Enabled and FCheckBox.Checked);
end;

procedure TFoxGroupBoxWithCheck.WMMove(var message:TMessage);
begin
  inherited;
  Invalidate;
  RepaintChildren;
end;

procedure TFoxGroupBoxWithCheck.WMEraseBkgnd(var message: TMessage);
begin
  if FTransparent then
    message.Result:= 1
  else
    inherited;
end;

constructor TFoxGroupBoxWithCheck.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCtl3D:= True;
  FCheckBox:= TFoxGroupCheck.Create(Self);
  FCheckBox.Parent:= Self;
  SetChecked(True);
  ControlStyle:= ControlStyle + [csAcceptsControls, csReplicatable];
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  TabStop:=True;
end;

destructor TFoxGroupBoxWithCheck.Destroy;
begin
  FCheckBox.Free;
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure TFoxPanel.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
    FCtl3D:= Value;
  Repaint;
end;

procedure TFoxPanel.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
    RepaintChildren;
  end;
end;

procedure TFoxPanel.SetTransparency(Value: Byte);
begin
  if value > 100 then value:= 100;
  if FTransparency <> Value then
  begin
    FTransparency:= Value;
    Invalidate;
    RepaintChildren;
  end;
end;

procedure TFoxPanel.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxPanel.paint;
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  R: TRect;
  Flags: Longint;
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor:= clBtnHighlight;
    if Bevel = bvLowered then TopColor:= clBtnShadow;
    BottomColor:= clBtnShadow;
    if Bevel = bvLowered then BottomColor:= clBtnHighlight;
  end;

begin
  if FTransparent then
  begin
    DrawParentImage(Self, Self.Canvas);
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Self.Color, FTransparency);
  end;

  with Canvas do
  begin
    Font:= Self.Font;
    R:= GetClientRect;
    if FCtl3D then
    begin
      // Draw outer bevel
      if BevelOuter <> bvNone then
      begin
        AdjustColors(BevelOuter);
        Frame3D(Canvas, R, TopColor, BottomColor, BevelWidth);
      end;
      // Draw frame between bevels
      Frame3D(Canvas, R, Color, Color, BorderWidth);
      // Draw inner bevel
      if BevelInner <> bvNone then
      begin
        AdjustColors(BevelInner);
        Frame3D(Canvas, R, TopColor, BottomColor, BevelWidth);
      end;
    end
    else
    begin
      Brush.Color:= clWindowFrame;
      FrameRect(R);
      InflateRect(R, -1, -1);
    end;
    Brush.Color:= Color;
    if not FTransparent then
      FillRect(R);
    Brush.Style:= bsClear;
    if Caption <> '' then
    begin
      with R do
      begin
        Top:= ((Bottom + Top) - TextHeight(Caption)) div 2;
        Bottom:= Top + TextHeight(Caption);
      end;
      Flags:= DT_EXPANDTABS or DT_VCENTER or Alignments[Alignment];
      Flags:= DrawTextBiDiModeFlags(Flags);
      DrawText(Handle, PChar(Caption), -1, R, Flags);
    end;
  end;
end;

procedure TFoxPanel.RepaintChildren;
var
  I : Integer;
begin
  if ControlCount > 0 then
  begin
    for I:=0 to ControlCount-1 do
      Controls[i].Invalidate;
  end;
end;

procedure TFoxPanel.CMEnabledChanged(var Message: TWmNoParams);
var
  I: Integer;
begin
  inherited;
  if ControlCount > 0 then
    for I:=0 to ControlCount-1 do
      Controls[i].Enabled:= Enabled;
end;

procedure TFoxPanel.WMMove(var message:TMessage);
begin
  inherited;
  RepaintChildren;
end;

procedure TFoxPanel.WMEraseBkgnd(var message: TWMERASEBKGND);
var
  canvas : TCanvas;
begin
  if FTransparent then
  begin
    canvas:= TCanvas.Create;
    canvas.handle := message.dc;
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Color, FTransparency);
    canvas.Free;
    message.Result:= 1
  end
  else
    inherited;
end;

constructor TFoxPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle + [csAcceptsControls, csReplicatable];
  FCtl3D:= True;
  FTransparent:= False;
  FTransparency:= 100;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  TabStop:=True;
end;

destructor TFoxPanel.Destroy;
begin
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('FoxControls', [TFoxCheckBox]);
  RegisterComponents('FoxControls', [TFoxRadioButton]);
  RegisterComponents('FoxControls', [TFoxGroupBox]);
  RegisterComponents('FoxControls', [TFoxGroupBoxWithCheck]);
  RegisterComponents('FoxControls', [TFoxRadioGroup]);
  RegisterComponents('FoxControls', [TFoxPanel]);
end;

end.


