unit FoxClasses;

// Part of the FoxControls package suite
// by A Fox Consulting & Design
// see FoxControls.txt or visit
// www.afox-consulting.com for more information.
//
// ************************************************
// ** Note: YOU MUST INCLUDE THIS UNIT INTO YOUR **
// ** APPLICATION'S 'USES' CLAUSE IF YOU REQUIRE **
// ** ACCESS TO ANY OF THE SPECIAL ENUMERATED    **
// ** PROPERTY CLASSES USED BY THE FOXCONTROLS.  **
// ************************************************

interface

uses Controls, Windows, Classes, Graphics, Messages;

type
  TOnMouseOverEvent = procedure(Sender: TObject) of object;
  TOnMouseOutEvent = procedure(Sender: TObject) of object;

  TFoxBorder=(fbNone,fbLowered,fbRaised,fbSimple);
  TFoxCheckType=(fcCheck,fcCross,fcCircle);
  TFoxCheckStyle=(fsFlat, fs3D, fs3DJumbo);
  TFoxBorderVisible=(bvAlways,bvWhenFocused);
  TGradStyle = (gsHorizontal, gsVertical);

  TRGB32 = packed record
    B, G, R, A: Byte;
  end;
  TColorArray = packed array[0..2047] of TRGB32;
  PColorArray = ^TColorArray;

procedure DrawParentImage(Control: TControl; Dest: TCanvas);
procedure ApplyAlphaBlending(Canvas: TCanvas; BaseColor: TColor; TranPercent: Byte);

implementation

procedure DrawParentImage(Control: TControl; Dest: TCanvas);
var
  SaveIndex: Integer;
  DC: HDC;
  Position: TPoint;
begin
  with Control do
  begin
    if Parent = nil then
      Exit;
    DC:= Dest.Handle;
    SaveIndex:= SaveDC(DC);
    GetViewportOrgEx(DC, Position);
    SetViewportOrgEx(DC, Position.X - Left, Position.Y - Top, nil);
    IntersectClipRect(DC, 0, 0, Parent.ClientWidth, Parent.ClientHeight);
    Parent.Perform(WM_ERASEBKGND, DC, 0);
    Parent.Perform(WM_PAINT, DC, 0);
    RestoreDC(DC, SaveIndex);
  end;
end;

procedure ApplyAlphaBlending(Canvas: TCanvas; BaseColor: TColor; TranPercent : Byte);
var
  x,y     : Integer;
  br,bg,bb: Byte;
  dr,dg,db: Integer;
  pline   : PColorArray;
  pc      : TRGB32;
  cb      : TBitmap;
begin
  BaseColor:= ColorToRGB(BaseColor);
  br:= (BaseColor and $FF);
  bg:= ((BaseColor and $FF00) shr 8);
  bb:= ((BaseColor and $FF0000) shr 16);
  cb:= TBitmap.Create;
  cb.PixelFormat:= pf32bit;
  cb.Width:= Canvas.ClipRect.Right - Canvas.ClipRect.Left;
  cb.Height:= Canvas.ClipRect.Bottom - Canvas.ClipRect.Top;
  BitBlt(cb.Canvas.Handle, 0, 0, cb.Width, cb.Height,
   Canvas.Handle, Canvas.ClipRect.Left, Canvas.ClipRect.Top, SRCCOPY);
  for y:= 0 to cb.Height - 1 do
  begin
    pline:= cb.ScanLine[y];
    for x:= 0 to cb.Width - 1 do
    begin
      pc:= pline[x];
      // Delta to color to
      dr:= br - pc.R;
      dg:= bg - pc.G;
      db:= bb - pc.B;
      pline[x].R:= br - ((dr * TranPercent) div 100);
      pline[x].G:= bg - ((dg * TranPercent) div 100);
      pline[x].B:= bb - ((db * TranPercent) div 100);
      pline[x].A:= 0;
    end;
  end;
  Canvas.Draw(Canvas.ClipRect.Left,Canvas.ClipRect.Top,cb);
  cb.Free;
end;

end.

