unit FoxEditControls;

// Part of the FoxControls package suite
// by A Fox Consulting & Design
// see FoxControls.txt or visit
// www.afox-consulting.com for more information.
//
// REMEMBER TO INCLUDE FoxClasses IN YOUR UNIT IF YOU NEED
// ACCESS TO THE SPECIAL ENUMERATED PROPERTY CLASSES USED
// BY THE FOXCONTROLS!

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms, StdCtrls, ExtCtrls, FoxClasses;

type
  TFoxEdit = class(TEdit)
  private
    SaveColor: TColor;
    SaveTransparent: Boolean;
    RedrawOnExit: Boolean;
    FAlignment: TAlignment;
    FFocusHighlight: Boolean;
    FHighlightColor: TColor;
    FTransparent: Boolean;
    FTransparency: Byte;
    FVersion: String;
    procedure SetAlignment(Value: TAlignment);
    procedure SetFocusHighlight(Value: boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparency(Value: Byte);
    procedure SetVersion(const Value: string);
    procedure CNCTLCOLOREDIT(var Message:TWMCTLCOLOREDIT); message CN_CTLCOLOREDIT;
    procedure CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC); message CN_CTLCOLORSTATIC;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure CreateWnd; override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Transparency: Byte read FTransparency write SetTransparency;
    property Version: string read FVersion write SetVersion;
  end;

type
  TFoxMemo = class(TMemo)
  private
    SaveColor: TColor;
    SaveTransparent: boolean;
    RedrawOnExit: Boolean;
    FAlignment: TAlignment;
    FFocusHighlight: Boolean;
    FHighlightColor: TColor;
    FTransparent: Boolean;
    FTransparency: Byte;
    FVersion:String;
    procedure SetAlignment(Value: TAlignment);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparency(Value: Byte);
    procedure SetVersion(const Value: string);
    procedure WMSetText(var Message:TWMSetText); message WM_SETTEXT;
    procedure CNCTLCOLOREDIT(var Message:TWMCTLCOLOREDIT); message CN_CTLCOLOREDIT;
    procedure CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC); message CN_CTLCOLORSTATIC;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    Procedure WMHScroll(var Message: TMessage ); message WM_HSCROLL;
    Procedure WMVScroll(var Message: TMessage ); message WM_VSCROLL;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure CreateWnd; override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Transparency: Byte read FTransparency write SetTransparency;
    property Version: string read FVersion write SetVersion;
  end;

  TFoxListBox = class(TListBox)
  private
    SaveColor: TColor;
    SaveTransparent: Boolean;
    RedrawOnExit: Boolean;
    FAlignment: TAlignment;
    FFocusHighlight: boolean;
    FHighlightColor: TColor;
    FTransparent: boolean;
    FTransparency: Byte;
    FVersion: String;
    procedure SetAlignment(Value: TAlignment);
    procedure SetFocusHighlight(Value: Boolean);
    procedure SetHighlightColor(Value: TColor);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparency(Value: Byte);
    procedure SetVersion(const Value: string);
    procedure CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC); message CN_CTLCOLORSTATIC;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    Procedure WMVScroll(var Message: TMessage ); message WM_VSCROLL;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMKillFocus); message WM_KILLFOCUS;
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight default False;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor default clLime;
    property Transparent: Boolean read FTransparent write SetTransparent default False;
    property Transparency: Byte read FTransparency write SetTransparency default 100;
    property Version: string read FVersion write SetVersion;
  end;

  TFoxStaticText = class(TCustomControl)
  private
    R,CaptRect: TRect;
    BiDiFlags: Longint;
    FAlignment: TAlignment;
    FAutoSize: Boolean;
    FBorder: TFoxBorder;
    FBorderColor:TColor;
    FTransparent: Boolean;
    FTransparency: Byte;
    FVersion: String;
    procedure SetAlignment(Value: TAlignment);
    procedure SetAutoSize(Value: boolean);
    procedure SetBorder(Value: TFoxBorder);
    procedure SetBorderColor(Value: TColor);
    procedure SetTransparent(Value: Boolean);
    procedure SetTransparency(Value: Byte);
    procedure SetVersion(const Value: String);
    procedure DrawBorder;
    procedure DrawCaption;
    procedure DrawCaptionDisabled;
    procedure CMEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CMParentColorChanged(var Message: TWMNoParams); message CM_PARENTCOLORCHANGED;
    procedure CMParentFontChanged(var Message: TWMNoParams); message CM_FONTCHANGED;
    procedure CMTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CMVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure WMMove(var Message: TMessage); message WM_MOVE;
  protected
    procedure Paint; override;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    //property AllowGrayed;  -- We do not support this feature
    property Anchors;
    property AutoSize: Boolean read FAutoSize write SetAutoSize;
    property BiDiMode;
    property Border: TFoxBorder read FBorder write SetBorder;
    property BorderColor: TColor read FBorderColor write SetBorderColor;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
    property ParentBiDiMode;
    property ParentFont;
    property ParentColor;
    property PopupMenu;
    property ShowHint;
    property TabStop;
    property TabOrder;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Transparency: Byte read FTransparency write SetTransparency default 100;
    property Version: String read FVersion write SetVersion;
    property Visible;
  end;

type
  TParentControl = class(TWinControl);

procedure Register;

implementation

procedure TFoxEdit.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TFoxEdit.SetFocusHighlight(Value: Boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxEdit.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
    FHighlightColor:= Value;
end;

procedure TFoxEdit.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
  end;
end;

procedure TFoxEdit.SetTransparency(Value: Byte);
begin
  if value > 100 then value:= 100;
  if FTransparency <> Value then
  begin
    FTransparency:= Value;
    Invalidate;
  end;
end;

procedure TFoxEdit.SetVersion(const Value: string);
begin
  FVersion:=FVersion;
end;

procedure TFoxEdit.CNCTLCOLOREDIT(var Message:TWMCTLCOLOREDIT);
begin
  if FTransparent then
  with Message do
    begin
      SetBkMode(ChildDC,Windows.TRANSPARENT);
      Result:=GetStockObject(HOLLOW_BRUSH)
    end
  else
    inherited;
end;

procedure TFoxEdit.CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC);
begin
  if FTransparent then
    with Message do
    begin
      SetBkMode(ChildDC,Windows.TRANSPARENT);
      Result:=GetStockObject(HOLLOW_BRUSH);
    end
  else
    inherited;
end;

procedure TFoxEdit.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    repaint
  else
    Invalidate;
end;

procedure TFoxEdit.WMEraseBkgnd(var Message: TWMERASEBKGND);
var
  canvas : TCanvas;
begin
  if FTransparent then
  begin
    canvas:= TCanvas.Create;
    canvas.handle := message.dc;
    DrawParentImage(Self, canvas);
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Color, FTransparency);
    canvas.Free;
  end
  else
    inherited;
end;

procedure TFoxEdit.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  SaveTransparent:= FTransparent;
  SaveColor:= Color;
  RedrawOnExit:= False;
  if FTransparent then
  begin
    FTransparent:= False;
    RedrawOnExit:= True;
  end;
  if FFocusHighlight then
  begin
    Color:= HighlightColor;
    RedrawOnExit:= True;
  end;
  Invalidate;
end;

procedure TFoxEdit.WMKillFocus(var Message: TWMKillFocus);
begin
  Inherited;
  FTransparent:= SaveTransparent;
  Color:= SaveColor;
  if RedrawOnExit then
    Invalidate;
  SetCursor(0);
end;

procedure TFoxEdit.CreateParams(var Params: TCreateParams);
begin
  inherited;
  case FAlignment of
    taLeftJustify: Params.Style   := Params.Style or ES_LEFT;
    taCenter: Params.Style := Params.Style or ES_CENTER;
    taRightJustify: Params.Style  := Params.Style or ES_RIGHT;
  end;
end;

procedure TFoxEdit.CreateWnd;
begin
  inherited CreateWnd;
  if fTransparent then
    begin
     SetWindowLong(Parent.Handle, GWL_STYLE,
     GetWindowLong(Parent.Handle, GWL_STYLE) and not WS_CLIPCHILDREN);
  end;
end;

constructor TFoxEdit.Create(Aowner: TComponent);
begin
  inherited Create(AOwner);
  FAlignment:= taLeftJustify;
  FFocusHighlight:= False;
  FHighlightColor:= clLime;
  FTransparent:=False;
  FTransparency:=100;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
end;

// ------------------------------------------------------------------------

procedure TFoxMemo.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TFoxMemo.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    invalidate;
  end;
end;

procedure TFoxMemo.SetTransparency(Value: Byte);
begin
  if value > 100 then value:= 100;
  if FTransparency <> Value then
  begin
    FTransparency:= Value;
    Invalidate;
  end;
end;

procedure TFoxMemo.SetFocusHighlight(Value: Boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxMemo.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
    FHighlightColor:= Value;
end;

procedure TFoxMemo.SetVersion(const Value: string);
begin
  FVersion:=FVersion;
end;

procedure TFoxMemo.WMSetText(var Message:TWMSetText);
begin
  inherited;
end;

procedure TFoxMemo.CNCTLCOLOREDIT(var Message:TWMCTLCOLOREDIT);
begin
  if FTransparent then
    with Message do
    begin
      SetBkMode(ChildDC,Windows.TRANSPARENT);
      Result:=GetStockObject(HOLLOW_BRUSH)
    end
  else
    inherited;
end;

procedure TFoxMemo.CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC);
begin
  if FTransparent then
    with Message do
    begin
      SetBkMode(ChildDC,Windows.TRANSPARENT);
      Result:=GetStockObject(HOLLOW_BRUSH);
    end
  else
    inherited;
end;

procedure TFoxMemo.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    repaint
  else
    Invalidate;
end;

procedure TFoxMemo.WMEraseBkgnd(var Message:TWMERASEBKGND);
var
  canvas : TCanvas;
begin
  if FTransparent then
  begin
    canvas:= TCanvas.Create;
    canvas.handle := message.dc;
    DrawParentImage(Self, canvas);
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Self.Color, FTransparency);
    canvas.Free;
  end
  else
    inherited;
end;

Procedure TFoxMemo.WMHScroll(var Message: TMessage);
begin
  if not Focused then
    SetFocus;
  inherited;
end;

Procedure TFoxMemo.WMVScroll(var Message: TMessage);
begin
  if not Focused then
    SetFocus;
  inherited;
end;

procedure TFoxMemo.CreateWnd;
begin
  inherited CreateWnd;
  if fTransparent then
    begin
     SetWindowLong(Parent.Handle, GWL_STYLE,
     GetWindowLong(Parent.Handle, GWL_STYLE) and not WS_CLIPCHILDREN);
  end;
end;

procedure TFoxMemo.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  SaveTransparent:= FTransparent;
  SaveColor:= Color;
  RedrawOnExit:= False;
  if FTransparent then
  begin
    FTransparent:= False;
    RedrawOnExit:= True;
  end;
  if FFocusHighlight then
  begin
    Color:= HighlightColor;
    RedrawOnExit:= True;
  end;
  Invalidate;
end;

procedure TFoxMemo.WMKillFocus(var Message: TWMKillFocus);
begin
  Inherited;
  FTransparent:= SaveTransparent;
  Color:= SaveColor;
  if RedrawOnExit then
    Invalidate;
  SetCursor(0);
end;

procedure TFoxMemo.CreateParams(var Params: TCreateParams);
begin
  inherited;
  case FAlignment of
    taLeftJustify: Params.Style   := Params.Style or ES_LEFT;
    taCenter: Params.Style := Params.Style or ES_CENTER;
    taRightJustify: Params.Style  := Params.Style or ES_RIGHT;
  end;
end;

constructor TFoxMemo.Create(Aowner: TComponent);
begin
  inherited Create(AOwner);
  FAlignment:= taLeftJustify;
  FFocusHighlight:= False;
  FHighlightColor:= clLime;
  FTransparent:=False;
  FTransparency:=100;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
end;

// ------------------------------------------------------------------------

procedure TFoxListBox.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    RecreateWnd;
  end;
end;

procedure TFoxListBox.SetFocusHighlight(Value: boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxListBox.SetHighlightColor(Value: TColor);
begin
  if FHighlightColor <> Value then
    FHighlightColor:= Value;
end;

procedure TFoxListBox.SetTransparent(Value: boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
  end;
end;

procedure TFoxListBox.SetTransparency(Value: byte);
begin
  if value > 100 then value:= 100;
  if FTransparency <> Value then
  begin
    FTransparency:= Value;
    Invalidate;
  end;
end;

procedure TFoxListBox.SetVersion(const Value: string);
begin
  FVersion:=FVersion;
end;

procedure TFoxListBox.CNCTLCOLORSTATIC(var Message: TWMCTLCOLORSTATIC);
begin
  if FTransparent then
    with Message do
    begin
      SetBkMode(ChildDC,Windows.TRANSPARENT);
      Result:=GetStockObject(HOLLOW_BRUSH);
    end
  else
    inherited;
end;

procedure TFoxListBox.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    repaint
  else
    Invalidate;
end;

procedure TFoxListBox.WMEraseBkgnd(var Message:TWMERASEBKGND);
var
  canvas : TCanvas;
begin
  if FTransparent then
  begin
    canvas:= TCanvas.Create;
    canvas.handle := message.dc;
    DrawParentImage(Self, canvas);
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Self.Color, FTransparency);
    canvas.Free;
  end
  else
    inherited;
end;

Procedure TFoxListBox.WMVScroll(var Message: TMessage);
begin
  if not Focused then
    SetFocus;
  inherited;
end;

procedure TFoxListBox.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  SaveTransparent:= FTransparent;
  SaveColor:= Color;
  RedrawOnExit:= False;
  if FTransparent then
  begin
    FTransparent:= False;
    RedrawOnExit:= True;
  end;
  if FFocusHighlight then
  begin
    Color:= HighlightColor;
    RedrawOnExit:= True;
  end;
  Invalidate;
end;

procedure TFoxListBox.WMKillFocus(var Message: TWMKillFocus);
begin
  Inherited;
  FTransparent:= SaveTransparent;
  Color:= SaveColor;
  if RedrawOnExit then
    Invalidate;
end;

procedure TFoxListBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
const
  Alignments: array[TAlignment] of Longint = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  FoundStyle: TBrushStyle;
  Flags: Longint;
begin
  if (not FTransparent) or (odSelected in State) then
    Self.Canvas.FillRect(Rect);

  if Index < Self.Items.Count then
  begin
    FoundStyle := Canvas.Brush.Style;       // Remember the brush style
    Flags:= DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX or Alignments[FAlignment]);
    if not UseRightToLeftAlignment then
      inc(Rect.Left, 2)
    else
      Dec(Rect.Right, 2);

    if FTransparent then
      if not (odSelected in State) then
        Canvas.Brush.Style := bsClear
      else
        Canvas.Brush.Style := bsSolid;

    DrawText(Self.Canvas.Handle, pChar(Self.Items[Index]), -1, Rect, Flags);
    Canvas.Brush.Style:= FoundStyle;
  end;
end;

procedure TFoxListBox.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);

  // TListBoxes are sorta stupid--they call WMEraseBkgnd *before* they
  // are moved/sized so a transparent background doesn't get updated.
  // To get around this we hide the listbox and then re-show it. This
  // forces a repaint at the current position.
  if (Parent <> nil) and IsWindowVisible(Handle) then
  begin
    ShowWindow(Handle, SW_HIDE);
    ShowWindow(Handle, SW_SHOW);
  end;
end;

constructor TFoxListBox.Create(Aowner: TComponent);
begin
  inherited Create(AOwner);
  FAlignment:= taLeftJustify;
  FFocusHighlight:= False;
  FHighlightColor:= clLime;
  FTransparent:=False;
  FTransparency:=100;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  Style:= lbOwnerDrawFixed;
end;

// ------------------------------------------------------------------------

procedure TFoxStaticText.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment:= Value;
    RecreateWnd;
  end;
end;

procedure TFoxStaticText.SetAutoSize(Value: Boolean);
begin
  if FAutoSize <> Value then
  begin
    FAutoSize:= Value;
    Invalidate;
  end;
end;

procedure TFoxStaticText.SetBorder(Value: TFoxBorder);
begin
  if FBorder <> Value then
  begin
    FBorder:= Value;
    Invalidate;
  end;
end;

procedure TFoxStaticText.SetBorderColor(Value: TColor);
begin
  if FBorderColor <> Value then
  begin
    FBorderColor:= Value;
    RecreateWnd;
  end;
end;

procedure TFoxStaticText.SetTransparent(Value: Boolean);
begin
  if FTransparent <> Value then
  begin
    FTransparent:= Value;
    Invalidate;
  end;
end;

procedure TFoxStaticText.SetTransparency(Value: Byte);
begin
  if value > 100 then value:= 100;
  if FTransparency <> Value then
  begin
    FTransparency:= Value;
    Invalidate;
  end;
end;

procedure TFoxStaticText.SetVersion(const Value: String);
begin
  FVersion:=FVersion;
end;

procedure TFoxStaticText.DrawBorder;
begin
  R:=clientRect;
  if not FTransparent then
  begin
    Canvas.Brush.Color:= Color;
    Canvas.Brush.Style:= bsSolid;
  end
  else
    Canvas.Brush.Style:= bsClear;
  case Border of
    fbNone: Frame3d(Canvas, R, clBtnFace,clBtnFace,0);
    fbLowered: Frame3d(Canvas, R,clBtnShadow,clBtnHighlight,1);
    fbRaised: Frame3d(Canvas, R,clBtnHighlight,clBtnShadow,1);
    fbSimple: Frame3d(Canvas, R,FBorderColor,FBorderColor,1);
  end;

  Canvas.FillRect(R);

  if FTransparent then Canvas.Brush.Style:= bsSolid;
end;

procedure TFoxStaticText.DrawCaption;
const
  Alignments: Array[TAlignment] of word = (DT_LEFT, DT_RIGHT, DT_CENTER);
begin
  with Canvas do
  begin
    Font:= Self.Font;
    Font.color:= Self.Font.Color;;

    Brush.Style:= bsClear;
    CaptRect:= ClientRect;
    if FBorder <> fbNone then
      InflateRect(CaptRect, -1, -1);

    if not Enabled then DrawCaptionDisabled;
    DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT or BiDiFlags or DT_WORDBREAK or Alignments[FAlignment]);
    Font.Color:= Self.Font.Color;
  end;
end;

procedure TFoxStaticText.DrawCaptionDisabled;
const
  Alignments: Array[TAlignment] of word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  ECaptRect: TRect;
begin
  with Canvas do
    begin
      Font:= Self.Font;
      brush.style:=bsClear;
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_LEFT or BiDiFlags or DT_WORDBREAK or Alignments[FAlignment]);
      font.Color :=clBtnHighlight;

      ECaptRect:=Rect(CaptRect.Left+1,CaptRect.top+1,CaptRect.Right+1,CaptRect.Bottom+1);
      DrawText(Handle, PChar(Caption), Length(Caption), ECaptRect, DT_LEFT or BiDiFlags or DT_WORDBREAK or Alignments[FAlignment]);
      Font.color :=clBtnShadow;
    end;
end;

procedure TFoxStaticText.CMEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxStaticText.CMParentColorChanged(var Message: TWMNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxStaticText.CMParentFontChanged(var Message: TWMNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxStaticText.CMTextChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxStaticText.CMVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Invalidate;
end;

procedure TFoxStaticText.WMMove(var message:TMessage);
begin
  inherited;
  if FTransparent then
    Invalidate;
end;

procedure TFoxStaticText.paint;
var
  NewWidth, NewHeight: Integer;
begin
  if FTransparent then
  begin
    DrawParentImage(Self, Self.Canvas);
    if FTransparency < 100 then
      ApplyAlphaBlending(canvas, Self.Color, FTransparency);
  end;

  DrawBorder;
  DrawCaption;
  if FAutoSize then
  begin
    NewWidth:= Canvas.TextWidth(Caption)+4;
    NewHeight:= Canvas.TextHeight(Caption)+4;
    Width:= NewWidth;
    Height:= NewHeight;
  end;
end;

constructor TFoxStaticText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle:= ControlStyle + [csReplicatable];
  SetBounds(0, 0, 65, 17);
  FAlignment:= taLeftJustify;
  FAutoSize:= True;
  FBorder:= fbNone;
  FBorderColor:= clBlue;
  FTransparency:= 100;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
  TabStop:=True;
end;

destructor TFoxStaticText.Destroy;
begin
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('FoxControls', [TFoxEdit]);
  RegisterComponents('FoxControls', [TFoxMemo]);
  RegisterComponents('FoxControls', [TFoxListBox]);
  RegisterComponents('FoxControls', [TFoxStaticText]);
end;

end.
