unit FoxMiscControls;

// Part of the FoxControls package suite
// by A Fox Consulting & Design
// see FoxControls.txt or visit
// www.afox-consulting.com for more information.
//
// REMEMBER TO INCLUDE FoxClasses IN YOUR UNIT IF YOU NEED
// ACCESS TO THE SPECIAL ENUMERATED PROPERTY CLASSES USED
// BY THE FOXCONTROLS!

interface

uses
  Windows, Messages, Classes, Graphics, Controls, StdCtrls, Buttons, ExtCtrls, FoxClasses;

type
  TFoxButton = class(TButton)
  private
    R,CaptRect,GRect: TRect;
    CurrentColor: TColor;
    CurrentFontColor: TColor;
    CurrentGlyph: TBitmap;
    FSq: Integer;  // Square size
    FGlyphCount: Integer;
    IsHot: Boolean;
    FCanvas: TCanvas;
    IsFocused: Boolean;
    FAlignment: TAlignment;
    FColor: TColor;
    FCtl3D: Boolean;
    FFocusHighlight: Boolean;
    FFocusRect: Boolean;
    FGlyph: TBitmap;
    FHighlightColor: TColor;
    FHotTrack: Boolean;
    FHotTrackFontColor: TColor;
    FLayout: TButtonLayout;
    FParentColor: Boolean;
    FShape: TShapeType;
    FImage: TBitmap;
    FVersion: String;
    FWordWrap: Boolean;
    procedure SetAlignment(Value: TAlignment);
    procedure SetColor(const Value: TColor);
    procedure SetCtl3D(Value: boolean);
    procedure SetFocusHighlight(Value: boolean);
    procedure SetFocusRect(Value: boolean);
    procedure SetGlyph(Value: TBitmap);
    procedure SetHighlightColor(const Value: TColor);
    procedure SetHotTrack(Value: Boolean);
    procedure SetHotTrackFontColor(const Value: TColor);
    procedure SetLayout(Value: TButtonLayout);
    procedure SetParentColor(Value: Boolean);
    procedure SetShape(Value: TShapeType);
    procedure SetImage(Value: TBitmap);
    procedure SetVersion(const Value: string);
    procedure SetWordWrap(Value: Boolean);
    procedure RedefineRegion;
    procedure DrawButton(MR: TRect; State: UINT);
    procedure DrawShapeRect(X1,Y1,X2,Y2: Integer);
    procedure DrawShapeFrame(fr: TRect; ff: Longint);
    procedure DrawCaption;
    procedure DrawCaptionDisabled;
    procedure PickGlyph;
    procedure DrawGlyph;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_PARENTCOLORCHANGED;
    procedure CNMeasureItem(var Message: TWMMeasureItem); message CN_MEASUREITEM;
    procedure CMMouseEnter(var AMsg: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var AMsg: TMessage); message CM_MOUSELEAVE;
    procedure CMTextChanged(var AMsg: TMessage); message CM_TEXTCHANGED;
    procedure CNDrawItem(var Message: TWMDrawItem); message CN_DRAWITEM;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure SetButtonStyle(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Color: TColor read FColor write SetColor;
    property Ctl3D: Boolean read FCtl3D write SetCtl3D;
    property FocusHighlight: Boolean read FFocusHighlight write SetFocusHighlight;
    property FocusRect: Boolean read FFocusRect write SetFocusRect;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property HighlightColor: TColor read FHighlightColor write SetHighlightColor;
    property HotTrack: Boolean read FHotTrack write SetHotTrack;
    property HotTrackFontColor: TColor read FHotTrackFontColor write SetHotTrackFontColor;
    property Layout: TButtonLayout read FLayout write SetLayout;
    property ParentColor: Boolean read FParentColor write SetParentColor;
    property Shape: TShapeType read FShape write SetShape;
    property Image: TBitmap read FImage write SetImage;
    property Version: string read FVersion write SetVersion;
    property WordWrap: Boolean read FWordWrap write SetWordWrap;
  end;

type
  TFoxBackground = class(TGraphicControl)
  private
    FTileImage: TBitmap;
    FGradientStartColor: TColor;
    FGradientEndColor: TColor;
    FStyle: TGradStyle;
    FVersion: String;
    procedure SetGradientEndColor(Value: TColor);
    procedure SetGradientStartColor(Value: TColor);
    procedure SetStyle(Value: TGradStyle);
    procedure SetTileImage(Value: TBitmap);
    procedure SetVersion(const Value: string);
    procedure DoGradientFill;
    procedure DoImageTile;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_EraseBkgnd;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property GradientEndColor: TColor read FGradientEndColor write SetGradientEndColor;
    property GradientStartColor: TColor read FGradientStartColor write SetGradientStartColor;
    property Style: TGradStyle read FStyle write SetStyle;
    property TileImage: TBitmap read FTileImage write SetTileImage;
    property Version: String read FVersion write SetVersion;
  end;

procedure Register;

implementation

procedure TFoxButton.SetAlignment(Value: TAlignment);
begin
  if FAlignment <> Value then
  begin
    FAlignment := Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetColor(const Value: TColor);
begin
  if FColor <> Value then
  begin
    FColor:= Value;
    FParentColor:= False;
    Invalidate;
  end;
end;

procedure TFoxButton.SetCtl3D(Value: Boolean);
begin
  if FCtl3D <> Value then
  begin
    FCtl3D:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetFocusHighlight(Value: boolean);
begin
  if FFocusHighlight <> Value then
    FFocusHighlight:= Value;
end;

procedure TFoxButton.SetFocusRect(Value: boolean);
begin
  if FFocusRect <> Value then
    FFocusRect:= Value;
end;

procedure TFoxButton.SetGlyph(Value: TBitmap);
var FirstTimeDraw: Boolean;
begin
  FirstTimeDraw:= (FGlyphCount = 0);
  FGlyph.Assign(Value);
  if not FGlyph.Empty then
  begin
    if FGlyph.Width mod FGlyph.Height = 0 then
      FGlyphCount:= FGlyph.Width div FGlyph.Height
    else
      FGlyphCount:= 1;
  end;

  if not FirstTimeDraw then
    Invalidate;
end;

procedure TFoxButton.SetHighlightColor(const Value: TColor);
begin
  if FHighlightColor <> Value then
  begin
    FHighlightColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetHotTrack(Value: Boolean);
begin
  if Value <> FHotTrack then
  begin
    FHotTrack:= Value;
  end;
end;

procedure TFoxButton.SetHotTrackFontColor(const Value: TColor);
begin
  if FHotTrackFontColor <> Value then
  begin
    FHotTrackFontColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetLayout(Value: TButtonLayout);
begin
  if Value <> FLayout then
  begin
    FLayout:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetParentColor(Value: Boolean);
begin
  if Value <> FParentColor then
  begin
    FParentColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.SetShape(Value: TShapeType);
begin
  if Value <> FShape then
  begin
    FShape:= Value;
    if FShape in [stSquare, stCircle] then
      SendMessage (Self.Handle, wm_Size, 0, 0)
    else
    begin
      Invalidate;
      RedefineRegion;
    end;
  end;
end;

procedure TFoxButton.SetImage(Value: TBitmap);
begin
  FImage.Assign(Value);
  Invalidate;
end;

procedure TFoxButton.SetVersion(const Value: string);
begin
  FVersion:=FVersion;
end;

procedure TFoxButton.SetWordWrap(Value: Boolean);
begin
  if FWordWrap <> Value then
  begin
    FWordWrap:= Value;
    Invalidate;
  end;
end;

procedure TFoxButton.RedefineRegion;
var
  W,H : Integer;
  rgn: HRGN;
begin
  W:= Self.Width;
  H:= Self.Height;
  if W < H then FSq:= W else FSq:= H;
  // Resize client rect working area to a aspect ratio of 1:1 if needed
  if FShape in [stSquare, stCircle, stRoundSquare] then
  begin
    if Self.Width <> FSq then Self.Width:= FSq;
    if Self.Height <> Fsq then Self.height:= FSq;
  end;

  if FShape in [stEllipse, stCircle] then
  begin
    rgn:= CreateEllipticRgn(0,0,Self.Width+1, Self.Height+1);
    SetWindowRgn(Self.Handle, rgn, True);
  end;
  if FShape in [stRoundRect, stRoundSquare] then
  begin
    rgn:= CreateRoundRectRgn(0,0,Self.Width+1, Self.Height+1, FSq div 2, FSq div 2);
    SetWindowRgn(Self.Handle, rgn, True);
  end;
  if FShape in [stRectangle, stSquare] then
  begin
    rgn:= CreateRectRgn(0,0,Self.Width, Self.Height);
    SetWindowRgn(Self.Handle, rgn, True);
  end;
end;

procedure TFoxButton.DrawButton(MR: TRect; State: UINT);
var
  Flags: Longint;
  IsDown, IsDisabled: Boolean;
  x,y  : integer;
begin
  Flags := DFCS_BUTTONPUSH or DFCS_ADJUSTRECT;
  IsDown := State and ODS_SELECTED <> 0;
  IsDisabled := State and ODS_DISABLED <> 0;

  if IsDown then Flags := Flags or DFCS_PUSHED;
  if IsDisabled then Flags := Flags or DFCS_INACTIVE;

  // Figure out colors
  if (Focused) and (FFocusHighlight) then
    CurrentColor:= FHighlightColor
  else
    CurrentColor:= FColor;
  if (IsHot) and (FHotTrack) then
    CurrentFontColor:= FHotTrackFontColor
  else
    CurrentFontColor:= Self.Font.Color;

  R:= MR; // Transfer client rect to shared private use

  FCanvas.Font.Color:= CurrentFontColor;

  // Paint background
  FCanvas.Brush.Color:= CurrentColor;
  FCanvas.FillRect(R);
  if IsDown then
    OffsetRect(R, 1, 1);  // bump down
  if not FImage.Empty then
  begin
    x:= R.Left + ((R.Right-R.Left) - FImage.Width) div 2;
    y:= R.Top + ((R.Bottom-R.Top) - FImage.Height) div 2;
    FCanvas.Draw(x, y, FImage);
  end;

  // Add focus highlight
  if (IsHot) and (FHotTrack) then
    ApplyAlphaBlending(FCanvas, clBtnHighlight, 50);

  if IsDown then
    OffsetRect(R, -1, -1); // bump back up

  // Draw Frame
  FCanvas.Brush.Style:= bsClear;
  if (Focused or Default) then
  begin
    FCanvas.Pen.Color := clWindowFrame;
    if not (FShape in [stRectangle, stSquare]) then
      FCanvas.Pen.Width := 2;
    DrawShapeRect(R.Left, R.Top, R.Right, R.Bottom);
    FCanvas.Pen.Width := 1;
    InflateRect(R, - 1, - 1);
  end;

  if IsDown then
  begin
    FCanvas.Pen.Color := clBtnShadow;
    FCanvas.Pen.Width := 1;
    DrawShapeRect(R.Left, R.Top, R.Right, R.Bottom);
  end
  else
    if FCtl3D then
      DrawShapeFrame(R, Flags)
    else
    begin
      FCanvas.Pen.Color := clWindowFrame;
      FCanvas.Pen.Width := 1;
      DrawShapeRect(R.Left, R.Top, R.Right, R.Bottom);
    end;

  if FGlyphCount = 0 then
    SetGlyph(FGlyph);

  // Determine size of caption/glyph rectangle
  R:= MR; // Start with original component rectangle
  if FShape in [stRectangle, stSquare] then
    InflateRect(R, -4, -4);
  if FShape in [stEllipse] then
    InflateRect(R, -2 -((R.Right-R.Left) div 6), -2 -((R.Bottom-R.Top) div 6));
  if FShape in [stCircle] then
    InflateRect(R, -2 -(FSq div 6), -2 -(FSq div 6));
  if FShape in [stRoundRect, stRoundSquare] then
    InflateRect(R, -2 -(FSq div 8), -2 -(FSq div 8));

  // Draw Focus Rectangle - Because the focus rectangle
  // can look a bit weird/ugly on elliptical buttons we
  // made drawing it configurable
  if (IsFocused) and (FFocusRect) then
  begin
    FCanvas.Pen.Color:= clWindowFrame;
    FCanvas.Brush.Color:= clBtnFace;
    DrawFocusRect(FCanvas.Handle, R);
  end;

  // Adjust caption positioning
  if IsDown then
    OffsetRect(R, 1, 0)
  else
    OffsetRect(R, 0, -1);

  PickGlyph;
  DrawCaption;
  DrawGlyph;
end;

procedure TFoxButton.DrawShapeRect(X1,Y1,X2,Y2: Integer);
begin
  if FShape in [stRectangle, stSquare] then
    FCanvas.Rectangle(X1,Y1,X2,Y2);
  if FShape in [stEllipse, stCircle] then
  begin
    FCanvas.Pen.Width:= 2;
    FCanvas.Ellipse(X1,Y1,X2,Y2);
    FCanvas.Pen.Width:= 1;
  end;
  if FShape in [stRoundRect, stRoundSquare] then
    FCanvas.RoundRect(X1,Y1,X2,Y2,FSq div 2,FSq div 2);
end;

procedure TFoxButton.DrawShapeFrame(fr: TRect; ff: Longint);
var
  ESize : Integer;
  fb    : TBitmap;
begin
  ESize:= (FSq div 2);

  // We'll draw the frame on a bitmap and copy it back to the
  // control's canvas. This reduces drawing flicker and also lets
  // HotTrack pattern to show through
  fb:= TBitmap.Create;
  with fb do
  begin
    Width:= fr.Right - fr.Left;
    Height:= fr.Bottom - fr.Top;
    Canvas.Brush.Color:= clBtnFace;
    Canvas.Brush.Style:= bsSolid;
    Canvas.FillRect(Rect(0, 0, Width, Height));

    if Focused then
      OffsetRect(fr, -1, -1);

    if FShape in [stRectangle, stSquare] then
    begin
      // May as well have Windows draw it if it's a standard shape
      DrawFrameControl(Canvas.Handle, Rect(fr.Left, fr.Top, fr.Right, fr.Bottom), DFC_BUTTON, ff);
    end;
    if FShape in [stRoundRect, stRoundSquare] then
    begin
      Canvas.Brush.Color:= clWindowFrame;
      Canvas.Pen.Color:= clWindowFrame;
      Canvas.RoundRect(fr.Left, fr.Top, fr.Right, fr.Bottom, ESize, ESize);
      Canvas.Brush.Color:= clBtnShadow;
      Canvas.Pen.Color:= clBtnShadow;
      Canvas.RoundRect(fr.Left+1, fr.Top+1, fr.Right-1, fr.Bottom-1, ESize, ESize);
      Canvas.Brush.Color:= clBtnHighlight;
      Canvas.Pen.Color:= clBtnHighlight;
      Canvas.RoundRect(fr.Left, fr.Top, fr.Right-2, fr.Bottom-2, ESize, ESize);
      Canvas.Brush.Color:= clBtnFace;
      Canvas.Pen.Color:= clBtnFace;
      Canvas.RoundRect(fr.Left+1, fr.Top+1, fr.Right-2, fr.Bottom-2, ESize, ESize);
    end;
    if FShape in [stEllipse, stCircle] then
    begin
      Canvas.Brush.Color:= clWindowFrame;
      Canvas.Pen.Color:= clWindowFrame;
      Canvas.Ellipse(fr.Left, fr.Top, fr.Right, fr.Bottom);
      Canvas.Brush.Color:= clBtnHighlight;
      Canvas.Pen.Color:= clBtnHighlight;
      Canvas.Ellipse(fr.Left, fr.Top, fr.Right-2, fr.Bottom-2);
      Canvas.Brush.Color:= clBtnShadow;
      Canvas.Pen.Color:= clBtnShadow;
      Canvas.Ellipse(fr.Left+1, fr.Top+1, fr.Right-1, fr.Bottom-1);
      Canvas.Brush.Color:= clBtnFace;
      Canvas.Pen.Color:= clBtnFace;
      Canvas.Ellipse(fr.Left+1, fr.Top+1, fr.Right-2, fr.Bottom-2);
    end;
  end;
  if Focused then
    OffsetRect(fr, 1, 1);

  FCanvas.Brush.Style:= bsClear;
  FCanvas.BrushCopy(fr, fb, Rect(0, 0, fb.Width, fb.Height), clBtnFace);
  FCanvas.Brush.Style:= bsSolid;
  fb.Free;
end;

procedure TFoxButton.DrawCaption;
const
  Alignments: Array[TAlignment] of word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  x,y,GlyphSizeX,GlyphSizeY,TextSizeX,TextSizeY : Integer;
  TCaption: String;
begin
  TCaption:= Caption;
  if pos('&', TCaption)>0 then
    Delete(TCaption, pos('&', TCaption), 1);

  with FCanvas do
  begin
    Brush.Style:= bsClear;

    if CurrentGlyph.Empty then
    begin
      GlyphSizeX:= 0;
      GlyphSizeY:= 0;
    end
    else
    begin
      GlyphSizeX:= CurrentGlyph.Width + 4;
      GlyphSizeY:= CurrentGlyph.Height + 4;
    end;

    // Calculate size available for caption
    if FWordWrap then
    begin
      TextSizeX:= (R.Right - R.Left);
      if (Layout = blGlyphLeft) or (Layout = blGlyphRight) then
        TextSizeX:= TextSizeX - GlyphSizeX;
      TextSizeY:= (R.Bottom - R.Top);
      if (Layout = blGlyphTop) or (Layout = blGlyphBottom) then
        TextSizeY:= TextSizeY - GlyphSizeY;
      // Shrink our caption area if necessary
      CaptRect:= Rect(0,0,TextSizeX, TextSizeY);
      DrawText(Handle, PChar(TCaption), Length(TCaption), CaptRect, DT_WORDBREAK or DT_CALCRECT or Alignments[FAlignment]);
      if (CaptRect.Bottom-CaptRect.Top) < TextSizeY then
        TextSizeY:= (CaptRect.Bottom-CaptRect.Top);
      if (CaptRect.Right-CaptRect.Left) < TextSizeX then
        TextSizeX:= (CaptRect.Right-CaptRect.Left);
    end
    else
    begin
      TextSizeX:= FCanvas.TextWidth(TCaption);
      TextSizeY:= FCanvas.TextHeight(TCaption);
    end;

    // Calculate position of caption
    case Layout of
      blGlyphRight: x:= R.Left + ((R.Right-R.Left) - TextSizeX - GlyphSizeX) div 2;
      blGlyphTop: x:= R.Left + ((R.Right-R.Left)  - TextSizeX) div 2;
      blGlyphBottom: x:= R.Left + ((R.Right-R.Left)  - TextSizeX) div 2;
    else
      // Default to blGlyphLeft
      x:= R.Left + ((R.Right-R.Left) - TextSizeX + GlyphSizeX) div 2;
    end;
    case Layout of
      blGlyphRight: y:= R.Top + ((R.Bottom-R.Top) - TextSizeY) div 2;
      blGlyphTop: y:= R.Top + ((R.Bottom-R.Top) - TextSizeY + GlyphSizeY) div 2;
      blGlyphBottom: y:= R.Top + ((R.Bottom-R.Top) - TextSizeY - GlyphSizeY) div 2;
    else
      // Default to blGlyphLeft
      y:= R.Top + ((R.Bottom-R.Top) - TextSizeY) div 2;
    end;
    CaptRect:= Rect(x, y, x+TextSizeX, y+TextSizeY);

    if not Enabled then DrawCaptionDisabled;
    if FWordWrap then
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_WORDBREAK or Alignments[FAlignment])
    else
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_SINGLELINE or Alignments[FAlignment]);
    Font.color:= Self.Font.Color;
    Brush.Style:= bsSolid;
  end;
end;

procedure TFoxButton.DrawCaptionDisabled;
const
  Alignments: Array[TAlignment] of word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  ECaptRect: TRect;
begin
  with FCanvas do
  begin
    Font:= Self.Font;
    Brush.Style:= bsClear;
    if FWordWrap then
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_WORDBREAK or Alignments[FAlignment])
    else
      DrawText(Handle, PChar(Caption), Length(Caption), CaptRect, DT_SINGLELINE or Alignments[FAlignment]);
    Font.Color:= clBtnHighlight;

    ECaptRect:= Rect(CaptRect.Left+1, CaptRect.Top+1, CaptRect.Right+1, CaptRect.Bottom+1);
    if FWordWrap then
      DrawText(Handle, PChar(Caption), Length(Caption), ECaptRect, DT_WORDBREAK or Alignments[FAlignment])
    else
      DrawText(Handle, PChar(Caption), Length(Caption), ECaptRect, DT_SINGLELINE or Alignments[FAlignment]);
    Font.Color:= clBtnShadow;
  end;
end;

// Sets up CurrentGlyph and GRest
procedure TFoxButton.PickGlyph;
var
  SourceRect: TRect;
begin
  if FGlyph.Empty then
  begin
    if not CurrentGlyph.Empty then
      CurrentGlyph.Assign(nil);
    exit;
  end;

  GRect:= Rect(0,0,FGlyph.Width div FGlyphCount,Fglyph.Height);
  if (FGlyphCount=2) and (not Enabled) then
    // If disabled and we have 2 glyphs, frame the second glyph
    SourceRect:= Rect(FGlyph.Width div FGlyphCount,0,FGlyph.Width,Fglyph.Height)
  else
    // Otherwise, Frame the first glyph in bitmap
    SourceRect:= GRect;

  CurrentGlyph.Width:= FGlyph.Width div FGlyphCount;
  CurrentGlyph.Height:= FGlyph.Height;
  CurrentGlyph.Canvas.CopyRect(GRect, FGlyph.canvas, SourceRect);
end;

procedure TFoxButton.DrawGlyph;
var
  DestRect: TRect;
  x, y : Integer;
  TransColor: TColor;
begin
  TransColor:=CurrentGlyph.Canvas.Pixels[0,CurrentGlyph.Height-1];

  // Calculate position of glyph
  case Layout of
    blGlyphRight: x:= R.Right + 2 - ((R.Right-R.Left) - (CaptRect.Right-CaptRect.Left) + CurrentGlyph.Width) div 2;
    blGlyphTop: x:= R.Left + ((R.Right-R.Left) - CurrentGlyph.Width) div 2;
    blGlyphBottom: x:= R.Left + ((R.Right-R.Left) - CurrentGlyph.Width) div 2;
  else
    // Default to blGlyphLeft
    x:= R.Left - 2 + ((R.Right-R.Left) - (CaptRect.Right-CaptRect.Left) - CurrentGlyph.Width) div 2;
  end;
  case Layout of
    blGlyphRight: y:= R.Top + ((R.Bottom-R.Top) - CurrentGlyph.Height) div 2;
    blGlyphTop: y:= R.Top - 2 + ((R.Bottom-R.Top) - (CaptRect.Bottom-CaptRect.Top) - CurrentGlyph.Height) div 2;
    blGlyphBottom: y:= R.Top - 2 + ((R.Bottom-R.Top) + (CaptRect.Bottom-CaptRect.Top) - CurrentGlyph.Height) div 2;
  else
    // Default to blGlyphLeft
    y:= R.Top + ((R.Bottom-R.Top) - CurrentGlyph.Height) div 2;
  end;
  DestRect:= Rect(x, y, x+CurrentGlyph.Width+1, y+CurrentGlyph.Height+1);
  FCanvas.Brush.Style:= bsClear;
  FCanvas.BrushCopy(DestRect, CurrentGlyph, GRect, TransColor);
end;

procedure TFoxButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TFoxButton.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TFoxButton.CMParentColorChanged(var Message: TMessage);
begin
  if FParentColor then
  begin
    if Message.wParam <> 0 then
      SetColor(TColor(Message.lParam))
    else
      SetColor(Parent.Brush.Color);
    FParentColor:= True;
  end;
end;

procedure TFoxButton.CMMouseEnter(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= True;
    repaint;
  end;
end;

procedure TFoxButton.CMMouseLeave(var AMsg: TMessage);
begin
  if (not enabled) or (csDesigning in ComponentState) then
    exit;

  if FHotTrack then
  begin
    IsHot:= False;
    repaint;
  end;
end;

procedure TFoxButton.CMTextChanged(var AMsg: TMessage);
begin
  Inherited;
  Invalidate;
end;

procedure TFoxButton.CNMeasureItem(var Message: TWMMeasureItem);
begin
  with Message.MeasureItemStruct^ do
  begin
    itemWidth  := Width;
    itemHeight := Height;
  end;
end;

procedure TFoxButton.CNDrawItem(var Message: TWMDrawItem);
var
  SaveIndex: Integer;
begin
  with Message.DrawItemStruct^ do
  begin
    SaveIndex := SaveDC(hDC);
    FCanvas.Lock;
    try
      FCanvas.Handle := hDC;
      FCanvas.Font := Font;
      FCanvas.Brush := Brush;
      DrawButton(rcItem, itemState);
    finally
      FCanvas.Handle := 0;
      FCanvas.Unlock;
      RestoreDC(hDC, SaveIndex);
    end;
  end;
  Message.Result := 1;
end;

procedure TFoxButton.WMSize(var Message: TWMSize);
begin
  RedefineRegion;
  inherited;
end;

procedure TFoxButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
    Style := Style or BS_OWNERDRAW;
end;

procedure TFoxButton.SetButtonStyle(Value: Boolean);
begin
  if Value <> IsFocused then
  begin
    IsFocused := Value;
    Invalidate;
  end;
end;

constructor TFoxButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCanvas:= TCanvas.Create;
  CurrentGlyph:= TBitmap.Create;
  FAlignment:= taCenter;
  FColor:= clBtnFace;
  FCtl3D:= True;
  FFocusRect:= True;
  FGlyph:= TBitmap.Create;
  FGlyphCount:= 0; // Will trigger internal call to SetGlypf
  FHighlightColor:= clLime;
  FHotTrack:= True;
  FHotTrackFontColor:= clBlue;
  FLayout:= blGlyphLeft;
  FParentColor:= False;
  FShape:= stRectangle;
  FImage:= TBitmap.Create;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
end;

destructor TFoxButton.Destroy;
begin
  FCanvas.Free;
  CurrentGlyph.Free;
  FGlyph.Free;
  FImage.Free;
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure TFoxBackground.SetGradientEndColor(Value: TColor);
begin
  if FGradientEndColor <> Value then
  begin
    FGradientEndColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxBackground.SetGradientStartColor(Value: TColor);
begin
  if FGradientStartColor <> Value then
  begin
    FGradientStartColor:= Value;
    Invalidate;
  end;
end;

procedure TFoxBackground.SetStyle(Value: TGradStyle);
begin
  if FStyle <> Value then
  begin
    FStyle:= Value;
    Invalidate;
  end;
end;

procedure TFoxBackground.SetTileImage(Value: TBitmap);
begin
  FTileImage.Assign(Value);
  Invalidate;
end;

procedure TFoxBackground.SetVersion(const Value: string);
begin
  FVersion:=FVersion;
end;

procedure TFoxBackground.DoGradientFill;
var
  dr, dg, db : Single;
  r, g, b    : Byte;
  x,y,step   : Smallint;
begin
  if FStyle = gsVertical then
    step:= Height
  else
    step:= Width;
  r:= ColorToRGB(FGradientStartColor) and $FF;
  g:= (ColorToRGB(FGradientStartColor) and $FF00) shr 8;
  b:= (ColorToRGB(FGradientStartColor) and $FF0000) shr 16;
  dr:= ((FGradientEndColor and $FF) - r) / step;
  dg:= (((FGradientEndColor and $FF00) shr 8) - g) / step;
  db:= (((FGradientEndColor and $FF0000) shr 16) - b) / step;
  with Canvas do
  begin
    if FStyle = gsVertical then
      for y:=0 to Height do
      begin
        Brush.Color:=
         ((b+trunc(db*y)) shl 16) +
         ((g+trunc(dg*y)) shl 8) +
          (r+trunc(dr*y));
        FillRect(Rect(0, y, Width, y+1));
      end
    else
      for x:=0 to Width do
      begin
        Brush.Color:=
         ((b+trunc(db*x)) shl 16) +
         ((g+trunc(dg*x)) shl 8) +
          (r+trunc(dr*x));
        FillRect(Rect(x, 0, x+1, Height));
      end;
  end;
end;

procedure TFoxBackground.DoImageTile;
var
  x, y: Integer;
begin
  for x := 0 to (Width div FTileImage.Width) do
    for y := 0 to (Height div FTileImage.Height) do
      Canvas.Draw(x * FTileImage.Width, y * FTileImage.Height, FTileImage);
end;

procedure TFoxBackground.Paint;
begin
  if not FTileImage.Empty then
    DoImageTile
  else
    DoGradientFill;
end;

procedure TFoxBackground.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

constructor TFoxBackground.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align:= alClient;
  FGradientStartColor:= clBtnFace;
  FGradientEndColor:= clWhite;
  FStyle := gsVertical;
  FTileImage:= TBitmap.Create;
  Width:=50;
  Height:=50;
  FVersion:='Version 2.00 by A Fox Consulting & Design';
end;

destructor TFoxBackground.Destroy;
begin
  FTileImage.Free;
  inherited Destroy;
end;

// ------------------------------------------------------------------------

procedure Register;
begin
  RegisterComponents('FoxControls', [TFoxButton]);
  RegisterComponents('FoxControls', [TFoxBackground]);
end;

end.


